@@ -21,6 +21,7 @@ from nemo.core.classes import ModelPT
 from nemo.core.classes.common import PretrainedModelInfo
 from nemo.core.classes.exportable import Exportable
 from nemo.core.classes.mixins import AccessMixin
+from nemo.core.utils.neural_type_utils import get_io_names
 from nemo.utils import logging, model_utils
 from nemo.utils.cast_utils import cast_all
 
@@ -83,15 +84,16 @@ class ASRModel(ModelPT, ABC):
 
             for loss_key, loss_registry in registry.items():
                 # Add auxiliary loss to total loss
-                loss_list = loss_registry['adapter_loss']
-                loss_value = sum(loss_list)
-                loss += loss_value
+                if 'adapter_loss' in loss_registry:
+                    loss_list = loss_registry['adapter_loss']
+                    loss_value = sum(loss_list)
+                    loss += loss_value
 
-                # Log current loss name and value
-                keys = loss_key.split(".")
-                key = "/".join(keys)
-                key = "adapter_loss/" + key
-                log_dict[key] = loss_value.detach()
+                    # Log current loss name and value
+                    keys = loss_key.split(".")
+                    key = "/".join(keys)
+                    key = "adapter_loss/" + key
+                    log_dict[key] = loss_value.detach()
 
             if len(log_dict) > 0:
                 self.log_dict(log_dict)
@@ -142,7 +144,7 @@ class ASRModel(ModelPT, ABC):
 
 class ExportableEncDecModel(Exportable):
     """
-    Simple utiliy mix-in to export models that consist of encoder/decoder pair 
+    Simple utiliy mix-in to export models that consist of encoder/decoder pair
     plus pre/post processor, but have to be exported as encoder/decoder pair only
     (covers most ASR classes)
     """
@@ -155,7 +157,19 @@ class ExportableEncDecModel(Exportable):
     def output_module(self):
         return self.decoder
 
-    def forward_for_export(self, input, length=None, cache_last_channel=None, cache_last_time=None):
+    @property
+    def output_names(self):
+        otypes = self.output_module.output_types
+        if hasattr(self.input_module, 'export_cache_support') and self.input_module.export_cache_support:
+            in_types = self.input_module.output_types
+            otypes = {n: t for (n, t) in list(otypes.items())[:1]}
+            for (n, t) in list(in_types.items())[1:]:
+                otypes[n] = t
+        return get_io_names(otypes, self.disabled_deployment_output_names)
+
+    def forward_for_export(
+        self, input, length=None, cache_last_channel=None, cache_last_time=None, cache_last_channel_len=None
+    ):
         """
         This forward is used when we need to export the model to ONNX format.
         Inputs cache_last_channel and cache_last_time are needed to be passed for exporting streaming models.
@@ -174,34 +188,53 @@ class ExportableEncDecModel(Exportable):
         """
         if hasattr(self.input_module, 'forward_for_export'):
             if cache_last_channel is None and cache_last_time is None:
-                encoder_output = self.input_module.forward_for_export(input, length)
+                encoder_output = self.input_module.forward_for_export(audio_signal=input, length=length)
             else:
                 encoder_output = self.input_module.forward_for_export(
-                    input, length, cache_last_channel, cache_last_time
+                    audio_signal=input,
+                    length=length,
+                    cache_last_channel=cache_last_channel,
+                    cache_last_time=cache_last_time,
+                    cache_last_channel_len=cache_last_channel_len,
                 )
         else:
             if cache_last_channel is None and cache_last_time is None:
-                encoder_output = self.input_module(input, length)
+                encoder_output = self.input_module(audio_signal=input, length=length)
             else:
-                encoder_output = self.input_module(input, length, cache_last_channel, cache_last_time)
+                encoder_output = self.input_module(
+                    audio_signal=input,
+                    length=length,
+                    cache_last_channel=cache_last_channel,
+                    cache_last_time=cache_last_time,
+                    cache_last_channel_len=cache_last_channel_len,
+                )
         if isinstance(encoder_output, tuple):
             decoder_input = encoder_output[0]
         else:
             decoder_input = encoder_output
         if hasattr(self.output_module, 'forward_for_export'):
             if cache_last_channel is None and cache_last_time is None:
-                ret = self.output_module.forward_for_export(decoder_input)
+                ret = self.output_module.forward_for_export(encoder_output=decoder_input)
             else:
-                # TODO: update this part to support full encoder/decoder export
-                ret = encoder_output
+                ret = self.output_module.forward_for_export(encoder_output=decoder_input)
         else:
             if cache_last_channel is None and cache_last_time is None:
-                ret = self.output_module(decoder_input)
+                ret = self.output_module(encoder_output=decoder_input)
             else:
-                # TODO: update this part to support full encoder/decoder export
-                ret = encoder_output
+                ret = self.output_module(encoder_output=decoder_input)
+        if cache_last_channel is None and cache_last_time is None:
+            pass
+        else:
+            if isinstance(ret, tuple):
+                ret = (ret[0], encoder_output[1], encoder_output[2], encoder_output[3], encoder_output[4])
+            else:
+                ret = (ret, encoder_output[1], encoder_output[2], encoder_output[3], encoder_output[4])
         return cast_all(ret, from_dtype=torch.float16, to_dtype=torch.float32)
 
     @property
     def disabled_deployment_input_names(self):
         return self.encoder.disabled_deployment_input_names
+
+    @property
+    def disabled_deployment_output_names(self):
+        return self.encoder.disabled_deployment_output_names
