@@ -25,7 +25,7 @@ from omegaconf import DictConfig, ListConfig, OmegaConf
 from pytorch_lightning import Trainer
 from torchmetrics.regression import MeanAbsoluteError, MeanSquaredError
 
-from nemo.collections.asr.data import audio_to_label_dataset
+from nemo.collections.asr.data import audio_to_label_dataset, feature_to_label_dataset
 from nemo.collections.asr.models.asr_model import ASRModel, ExportableEncDecModel
 from nemo.collections.asr.parts.preprocessing.features import WaveformFeaturizer
 from nemo.collections.asr.parts.preprocessing.perturb import process_augmentations
@@ -124,8 +124,10 @@ class _EncDecBaseModel(ASRModel, ExportableEncDecModel):
         else:
             audio_eltype = AudioSignal()
         return {
-            "input_signal": NeuralType(('B', 'T'), audio_eltype),
-            "input_signal_length": NeuralType(tuple('B'), LengthsType()),
+            "input_signal": NeuralType(('B', 'T'), audio_eltype, optional=True),
+            "input_signal_length": NeuralType(tuple('B'), LengthsType(), optional=True),
+            "processed_signal": NeuralType(('B', 'D', 'T'), SpectrogramType(), optional=True),
+            "processed_signal_length": NeuralType(tuple('B'), LengthsType(), optional=True),
         }
 
     @property
@@ -133,19 +135,30 @@ class _EncDecBaseModel(ASRModel, ExportableEncDecModel):
     def output_types(self) -> Optional[Dict[str, NeuralType]]:
         pass
 
-    def forward(self, input_signal, input_signal_length):
-        processed_signal, processed_signal_len = self.preprocessor(
-            input_signal=input_signal, length=input_signal_length,
-        )
+    def forward(
+        self, input_signal=None, input_signal_length=None, processed_signal=None, processed_signal_length=None
+    ):
+        has_input_signal = input_signal is not None and input_signal_length is not None
+        has_processed_signal = processed_signal is not None and processed_signal_length is not None
+        if (has_input_signal ^ has_processed_signal) == False:
+            raise ValueError(
+                f"{self} Arguments ``input_signal`` and ``input_signal_length`` are mutually exclusive "
+                " with ``processed_signal`` and ``processed_signal_length`` arguments."
+            )
+
+        if not has_processed_signal:
+            processed_signal, processed_signal_length = self.preprocessor(
+                input_signal=input_signal, length=input_signal_length,
+            )
         # Crop or pad is always applied
         if self.crop_or_pad is not None:
-            processed_signal, processed_signal_len = self.crop_or_pad(
-                input_signal=processed_signal, length=processed_signal_len
+            processed_signal, processed_signal_length = self.crop_or_pad(
+                input_signal=processed_signal, length=processed_signal_length
             )
         # Spec augment is not applied during evaluation/testing
         if self.spec_augmentation is not None and self.training:
-            processed_signal = self.spec_augmentation(input_spec=processed_signal, length=processed_signal_len)
-        encoded, encoded_len = self.encoder(audio_signal=processed_signal, length=processed_signal_len)
+            processed_signal = self.spec_augmentation(input_spec=processed_signal, length=processed_signal_length)
+        encoded, encoded_len = self.encoder(audio_signal=processed_signal, length=processed_signal_length)
         logits = self.decoder(encoder_output=encoded)
         return logits
 
@@ -179,14 +192,17 @@ class _EncDecBaseModel(ASRModel, ExportableEncDecModel):
 
         self._validation_dl = self._setup_dataloader_from_config(config=DictConfig(val_data_config))
 
-    def setup_test_data(self, test_data_config: Optional[Union[DictConfig, Dict]]):
+    def setup_test_data(self, test_data_config: Optional[Union[DictConfig, Dict]], use_feat: bool = False):
         if 'shuffle' not in test_data_config:
             test_data_config['shuffle'] = False
 
         # preserve config
         self._update_dataset_config(dataset_name='test', config=test_data_config)
 
-        self._test_dl = self._setup_dataloader_from_config(config=DictConfig(test_data_config))
+        if use_feat:
+            self._test_dl = self._setup_feature_label_dataloader(config=DictConfig(test_data_config))
+        else:
+            self._test_dl = self._setup_dataloader_from_config(config=DictConfig(test_data_config))
 
     def test_dataloader(self):
         if self._test_dl is not None:
@@ -266,6 +282,43 @@ class _EncDecBaseModel(ASRModel, ExportableEncDecModel):
             pin_memory=config.get('pin_memory', False),
         )
 
+    def _setup_feature_label_dataloader(self, config: DictConfig) -> torch.utils.data.DataLoader:
+        """
+        setup dataloader for VAD inference with audio features as input
+        """
+
+        OmegaConf.set_struct(config, False)
+        config.is_regression_task = self.is_regression_task
+        OmegaConf.set_struct(config, True)
+
+        if 'augmentor' in config:
+            augmentor = process_augmentations(config['augmentor'])
+        else:
+            augmentor = None
+        if 'manifest_filepath' in config and config['manifest_filepath'] is None:
+            logging.warning(f"Could not load dataset as `manifest_filepath` is None. Provided config : {config}")
+            return None
+
+        dataset = feature_to_label_dataset.get_feature_label_dataset(config=config, augmentor=augmentor)
+        if 'vad_stream' in config and config['vad_stream']:
+            collate_func = dataset._vad_segment_collate_fn
+            batch_size = 1
+            shuffle = False
+        else:
+            collate_func = dataset._collate_fn
+            batch_size = config['batch_size']
+            shuffle = config['shuffle']
+
+        return torch.utils.data.DataLoader(
+            dataset=dataset,
+            batch_size=batch_size,
+            collate_fn=collate_func,
+            drop_last=config.get('drop_last', False),
+            shuffle=shuffle,
+            num_workers=config.get('num_workers', 0),
+            pin_memory=config.get('pin_memory', False),
+        )
+
     @torch.no_grad()
     def transcribe(self, paths2audio_files: List[str], batch_size: int = 4, logprobs=False) -> List[str]:
         """
@@ -497,7 +550,7 @@ class EncDecClassificationModel(_EncDecBaseModel):
         self._accuracy.reset()
 
         for top_k, score in zip(self._accuracy.top_k, topk_scores):
-            self.log('training_batch_accuracy_top@{}'.format(top_k), score)
+            self.log('training_batch_accuracy_top_{}'.format(top_k), score)
 
         return {
             'loss': loss_value,
@@ -562,8 +615,15 @@ class EncDecClassificationModel(_EncDecBaseModel):
         return {'log': tensorboard_log}
 
     @typecheck()
-    def forward(self, input_signal, input_signal_length):
-        logits = super().forward(input_signal=input_signal, input_signal_length=input_signal_length)
+    def forward(
+        self, input_signal=None, input_signal_length=None, processed_signal=None, processed_signal_length=None
+    ):
+        logits = super().forward(
+            input_signal=input_signal,
+            input_signal_length=input_signal_length,
+            processed_signal=processed_signal,
+            processed_signal_length=processed_signal_length,
+        )
         return logits
 
     def change_labels(self, new_labels: List[str]):
