@@ -97,6 +97,7 @@ class EncDecHybridRNNTCTCModel(EncDecRNNTModel, ASRBPEMixin):
         partial_hypothesis: Optional[List['Hypothesis']] = None,
         num_workers: int = 0,
         channel_selector: Optional[ChannelSelectorType] = None,
+        augmentor: DictConfig = None,
     ) -> (List[str], Optional[List['Hypothesis']]):
         """
         Uses greedy decoding to transcribe audio files. Use this method for debugging and prototyping.
@@ -112,6 +113,7 @@ class EncDecHybridRNNTCTCModel(EncDecRNNTModel, ASRBPEMixin):
         With hypotheses can do some postprocessing like getting timestamp or rescoring
             num_workers: (int) number of workers for DataLoader
             channel_selector (int | Iterable[int] | str): select a single channel or a subset of channels from multi-channel audio. If set to `'average'`, it performs averaging across channels. Disabled if set to `None`. Defaults to `None`. Uses zero-based indexing.
+            augmentor: (DictConfig): Augment audio samples during transcription if augmentor is applied.
 
         Returns:
             Returns a tuple of 2 items -
@@ -119,7 +121,15 @@ class EncDecHybridRNNTCTCModel(EncDecRNNTModel, ASRBPEMixin):
             * An optional list of beam search transcript texts / Hypothesis / NBestHypothesis.
         """
         if self.use_rnnt_decoder:
-            return super().transcribe(**kwargs)
+            return super().transcribe(
+                paths2audio_files=paths2audio_files,
+                batch_size=batch_size,
+                return_hypotheses=return_hypotheses,
+                partial_hypothesis=partial_hypothesis,
+                num_workers=num_workers,
+                channel_selector=channel_selector,
+                augmentor=augmentor,
+            )
 
         if paths2audio_files is None or len(paths2audio_files) == 0:
             return {}
@@ -165,6 +175,9 @@ class EncDecHybridRNNTCTCModel(EncDecRNNTModel, ASRBPEMixin):
                     'channel_selector': channel_selector,
                 }
 
+                if augmentor:
+                    config['augmentor'] = augmentor
+
                 temporary_datalayer = self._setup_transcribe_dataloader(config)
                 for test_batch in tqdm(temporary_datalayer, desc="Transcribing"):
                     encoded, encoded_len = self.forward(
@@ -178,7 +191,7 @@ class EncDecHybridRNNTCTCModel(EncDecRNNTModel, ASRBPEMixin):
                     if return_hypotheses:
                         # dump log probs per file
                         for idx in range(logits.shape[0]):
-                            best_hyp[idx].y_sequence = logits[idx][: logits_len[idx]]
+                            best_hyp[idx].y_sequence = logits[idx][: encoded_len[idx]]
                             if best_hyp[idx].alignments is None:
                                 best_hyp[idx].alignments = best_hyp[idx].y_sequence
                     del logits
