@@ -15,7 +15,7 @@
 import math
 import os
 from dataclasses import dataclass
-from typing import Iterable, List, Optional, Tuple, Union
+from typing import List, Optional, Tuple, Union
 
 import torch
 
@@ -25,6 +25,8 @@ from nemo.core.classes import Typing, typecheck
 from nemo.core.neural_types import HypothesisType, LengthsType, LogprobsType, NeuralType
 from nemo.utils import logging
 
+DEFAULT_TOKEN_OFFSET = 100
+
 
 def pack_hypotheses(
     hypotheses: List[rnnt_utils.NBestHypotheses], logitlen: torch.Tensor,
@@ -98,6 +100,10 @@ class AbstractBeamCTCInfer(Typing):
         self.decoding_type = None
         self.tokenizer = None
 
+        # Utility maps for vocabulary
+        self.vocab_index_map = None
+        self.index_vocab_map = None
+
         # Internal variable, used to prevent double reduction of consecutive tokens (ctc collapse)
         self.override_fold_consecutive_value = None
 
@@ -110,6 +116,8 @@ class AbstractBeamCTCInfer(Typing):
                 Note that this vocabulary must NOT contain the "BLANK" token.
         """
         self.vocab = vocab
+        self.vocab_index_map = {v: i for i, v in enumerate(vocab)}
+        self.index_vocab_map = {i: v for i, v in enumerate(vocab)}
 
     def set_decoding_type(self, decoding_type: str):
         """
@@ -186,7 +194,7 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
         beam_beta: float = 0.0,
         kenlm_path: str = None,
         flashlight_cfg: Optional['FlashlightConfig'] = None,
-        # pyctcdecode_cfg: Optional['PyCTCDecodeConfig'] = None,
+        pyctcdecode_cfg: Optional['PyCTCDecodeConfig'] = None,
     ):
         super().__init__(blank_id=blank_id, beam_size=beam_size)
 
@@ -204,8 +212,6 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
             self.search_algorithm = self.default_beam_search
         elif search_type == "pyctcdecode":
             self.search_algorithm = self._pyctcdecode_beam_search
-
-            raise NotImplementedError(f"The search type of `pyctcdecode` is currently not supported.\n" f"")
         elif search_type == "flashlight":
             self.search_algorithm = self.flashlight_beam_search
         else:
@@ -214,6 +220,9 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
                 f"Please use one of : (default, nemo, pyctcdecode)"
             )
 
+        # Log the beam search algorithm
+        logging.info(f"Beam search algorithm: {search_type}")
+
         self.beam_alpha = beam_alpha
         self.beam_beta = beam_beta
 
@@ -221,9 +230,9 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
         self.kenlm_path = kenlm_path
 
         # PyCTCDecode params
-        # if pyctcdecode_cfg is None:
-        #     pyctcdecode_cfg = PyCTCDecodeConfig()
-        # self.pyctcdecode_cfg = pyctcdecode_cfg  # type: PyCTCDecodeConfig
+        if pyctcdecode_cfg is None:
+            pyctcdecode_cfg = PyCTCDecodeConfig()
+        self.pyctcdecode_cfg = pyctcdecode_cfg  # type: PyCTCDecodeConfig
 
         if flashlight_cfg is None:
             flashlight_cfg = FlashlightConfig()
@@ -256,7 +265,7 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
         if self.decoding_type is None:
             raise ValueError("Please set the decoding type with `set_decoding_type()` before calling this function.")
 
-        with torch.inference_mode():
+        with torch.no_grad(), torch.inference_mode():
             # Process each sequence independently
             prediction_tensor = decoder_output
 
@@ -284,13 +293,15 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
         self, x: torch.Tensor, out_len: torch.Tensor
     ) -> List[Union[rnnt_utils.Hypothesis, rnnt_utils.NBestHypotheses]]:
         """
+        Open Seq2Seq Beam Search Algorithm (DeepSpeed)
 
         Args:
-            x: Tensor of shape [B, T, V+1]
-            out_len:
+            x: Tensor of shape [B, T, V+1], where B is the batch size, T is the maximum sequence length,
+                and V is the vocabulary size. The tensor contains log-probabilities.
+            out_len: Tensor of shape [B], contains lengths of each sequence in the batch.
 
         Returns:
-
+            A list of NBestHypotheses objects, one for each sequence in the batch.
         """
         if self.compute_timestamps:
             raise ValueError(
@@ -349,7 +360,8 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
                 if self.decoding_type == 'subword':
                     pred_token_ids = [ord(c) - self.token_offset for c in candidate[1]]
                 else:
-                    pred_token_ids = candidate[1]
+                    # Char models
+                    pred_token_ids = [self.vocab_index_map[c] for c in candidate[1]]
 
                 # We preserve the token ids and the score for this hypothesis
                 hypothesis.y_sequence = pred_token_ids
@@ -374,81 +386,106 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
     def _pyctcdecode_beam_search(
         self, x: torch.Tensor, out_len: torch.Tensor
     ) -> List[Union[rnnt_utils.Hypothesis, rnnt_utils.NBestHypotheses]]:
-        logging.warning("pyctcdecode is not yet integrated.")
+        """
+        PyCTCDecode Beam Search Algorithm. Should support Char and Subword models.
 
-        raise NotImplementedError(
-            "Currently, pyctcdecode has not ben formally integrated into the decoding framework."
-        )
+        Args:
+            x: Tensor of shape [B, T, V+1], where B is the batch size, T is the maximum sequence length,
+                and V is the vocabulary size. The tensor contains log-probabilities.
+            out_len: Tensor of shape [B], contains lengths of each sequence in the batch.
 
-        # try:
-        #     import pyctcdecode
-        # except (ImportError, ModuleNotFoundError):
-        #     raise ImportError(
-        #         f"Could not load `pyctcdecode` library. Please install it from pip using :\n"
-        #         f"pip install --upgrade pyctcdecode"
-        #     )
-        #
-        # if self.pyctcdecode_beam_scorer is None:
-        #     self.pyctcdecode_beam_scorer = pyctcdecode.build_ctcdecoder(
-        #         labels=self.vocab, kenlm_model_path=self.kenlm_path, alpha=self.beam_alpha, beta=self.beam_beta
-        #     )  # type: pyctcdecode.BeamSearchDecoderCTC
-        #
-        # x = x.to('cpu')
-        #
-        # with typecheck.disable_checks():
-        #     beams_batch = []
-        #     for sample_id in range(len(x)):
-        #         logprobs = x[sample_id, : out_len[sample_id], :]
-        #         result = self.pyctcdecode_beam_scorer.decode_beams(
-        #             logprobs,
-        #             beam_width=self.beam_size,
-        #             beam_prune_logp=self.pyctcdecode_cfg.beam_prune_logp,
-        #             token_min_logp=self.pyctcdecode_cfg.token_min_logp,
-        #             prune_history=self.pyctcdecode_cfg.prune_history,
-        #             hotwords=self.pyctcdecode_cfg.hotwords,
-        #             hotword_weight=self.pyctcdecode_cfg.hotword_weight,
-        #             lm_start_state=None,
-        #         )  # Output format: text, last_lm_state, text_frames, logit_score, lm_score
-        #         beams_batch.append(result)
-        #
-        # nbest_hypotheses = []
-        # for beams_idx, beams in enumerate(beams_batch):
-        #     hypotheses = []
-        #     for candidate_idx, candidate in enumerate(beams):
-        #         # Candidate = (text, last_lm_state, text_frames, logit_score, lm_score)
-        #         hypothesis = rnnt_utils.Hypothesis(
-        #             score=0.0, y_sequence=[], dec_state=None, timestep=[], last_token=None
-        #         )
-        #
-        #         # TODO: Requires token ids to be returned rather than text.
-        #         pred_token_ids = None
-        #
-        #         hypothesis.y_sequence = pred_token_ids
-        #         hypothesis.text = candidate[0]  # text
-        #         hypothesis.score = candidate[4]  # score
-        #
-        #         if self.preserve_alignments:
-        #             hypothesis.alignments = x[beams_idx][: out_len[beams_idx]]
-        #
-        #         hypotheses.append(hypothesis)
-        #
-        #     hypotheses = rnnt_utils.NBestHypotheses(hypotheses)
-        #     nbest_hypotheses.append(hypotheses)
-        #
-        # return nbest_hypotheses
+        Returns:
+            A list of NBestHypotheses objects, one for each sequence in the batch.
+        """
+        if self.compute_timestamps:
+            raise ValueError(
+                f"Beam Search with strategy `{self.search_type}` does not support time stamp calculation!"
+            )
+
+        try:
+            import pyctcdecode
+        except (ImportError, ModuleNotFoundError):
+            raise ImportError(
+                f"Could not load `pyctcdecode` library. Please install it from pip using :\n"
+                f"pip install --upgrade pyctcdecode"
+            )
+
+        if self.pyctcdecode_beam_scorer is None:
+            self.pyctcdecode_beam_scorer = pyctcdecode.build_ctcdecoder(
+                labels=self.vocab, kenlm_model_path=self.kenlm_path, alpha=self.beam_alpha, beta=self.beam_beta
+            )  # type: pyctcdecode.BeamSearchDecoderCTC
+
+        x = x.to('cpu').numpy()
+
+        with typecheck.disable_checks():
+            beams_batch = []
+            for sample_id in range(len(x)):
+                logprobs = x[sample_id, : out_len[sample_id], :]
+                result = self.pyctcdecode_beam_scorer.decode_beams(
+                    logprobs,
+                    beam_width=self.beam_size,
+                    beam_prune_logp=self.pyctcdecode_cfg.beam_prune_logp,
+                    token_min_logp=self.pyctcdecode_cfg.token_min_logp,
+                    prune_history=self.pyctcdecode_cfg.prune_history,
+                    hotwords=self.pyctcdecode_cfg.hotwords,
+                    hotword_weight=self.pyctcdecode_cfg.hotword_weight,
+                    lm_start_state=None,
+                )  # Output format: text, last_lm_state, text_frames, logit_score, lm_score
+                beams_batch.append(result)
+
+        nbest_hypotheses = []
+        for beams_idx, beams in enumerate(beams_batch):
+            hypotheses = []
+            for candidate_idx, candidate in enumerate(beams):
+                # Candidate = (text, last_lm_state, text_frames, logit_score, lm_score)
+                hypothesis = rnnt_utils.Hypothesis(
+                    score=0.0, y_sequence=[], dec_state=None, timestep=[], last_token=None
+                )
+
+                # TODO: Requires token ids to be returned rather than text.
+                if self.decoding_type == 'subword':
+                    if self.tokenizer is None:
+                        raise ValueError("Tokenizer must be provided for subword decoding. Use set_tokenizer().")
+
+                    pred_token_ids = self.tokenizer.text_to_ids(candidate[0])
+                else:
+                    if self.vocab is None:
+                        raise ValueError("Vocab must be provided for character decoding. Use set_vocab().")
+
+                    chars = list(candidate[0])
+                    pred_token_ids = [self.vocab_index_map[c] for c in chars]
+
+                hypothesis.y_sequence = pred_token_ids
+                hypothesis.text = candidate[0]  # text
+                hypothesis.score = candidate[4]  # score
+
+                # Inject word level timestamps
+                hypothesis.timestep = candidate[2]  # text_frames
+
+                if self.preserve_alignments:
+                    hypothesis.alignments = torch.from_numpy(x[beams_idx][: out_len[beams_idx]])
+
+                hypotheses.append(hypothesis)
+
+            hypotheses = rnnt_utils.NBestHypotheses(hypotheses)
+            nbest_hypotheses.append(hypotheses)
+
+        return nbest_hypotheses
 
     @torch.no_grad()
     def flashlight_beam_search(
         self, x: torch.Tensor, out_len: torch.Tensor
     ) -> List[Union[rnnt_utils.Hypothesis, rnnt_utils.NBestHypotheses]]:
         """
+        Flashlight Beam Search Algorithm. Should support Char and Subword models.
 
         Args:
-            x: Tensor of shape [B, T, V+1]
-            out_len:
+            x: Tensor of shape [B, T, V+1], where B is the batch size, T is the maximum sequence length,
+                and V is the vocabulary size. The tensor contains log-probabilities.
+            out_len: Tensor of shape [B], contains lengths of each sequence in the batch.
 
         Returns:
-
+            A list of NBestHypotheses objects, one for each sequence in the batch.
         """
         if self.compute_timestamps:
             raise ValueError(
@@ -528,10 +565,9 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
         # Please check train_kenlm.py in scripts/asr_language_modeling/ to find out why we need
         # TOKEN_OFFSET for BPE-based models
         if self.decoding_type == 'subword':
-            self.token_offset = 100
+            self.token_offset = DEFAULT_TOKEN_OFFSET
 
 
-# NOTE: Unused as of now
 @dataclass
 class PyCTCDecodeConfig:
     # These arguments cannot be imported from pyctcdecode (optional dependency)
@@ -540,7 +576,7 @@ class PyCTCDecodeConfig:
     beam_prune_logp: float = -10.0
     token_min_logp: float = -5.0
     prune_history: bool = False
-    hotwords: Optional[Iterable[str]] = None
+    hotwords: Optional[List[str]] = None
     hotword_weight: float = 10.0
 
 
@@ -566,5 +602,5 @@ class BeamCTCInferConfig:
     beam_beta: float = 0.0
     kenlm_path: Optional[str] = None
 
-    # pyctcdecode_cfg: PyCTCDecodeConfig = PyCTCDecodeConfig()
     flashlight_cfg: Optional[FlashlightConfig] = FlashlightConfig()
+    pyctcdecode_cfg: Optional[PyCTCDecodeConfig] = PyCTCDecodeConfig()
