@@ -28,7 +28,7 @@
 
 import copy
 from dataclasses import dataclass
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import numpy as np
 import torch
@@ -40,6 +40,13 @@ from nemo.core.classes import Typing, typecheck
 from nemo.core.neural_types import AcousticEncodedRepresentation, HypothesisType, LengthsType, NeuralType
 from nemo.utils import logging
 
+try:
+    import kenlm
+
+    KENLM_AVAILABLE = True
+except (ImportError, ModuleNotFoundError):
+    KENLM_AVAILABLE = False
+
 
 def pack_hypotheses(hypotheses: List[Hypothesis]) -> List[Hypothesis]:
     for idx, hyp in enumerate(hypotheses):  # type: rnnt_utils.Hypothesis
@@ -48,6 +55,10 @@ def pack_hypotheses(hypotheses: List[Hypothesis]) -> List[Hypothesis]:
         if hyp.dec_state is not None:
             hyp.dec_state = _states_to_device(hyp.dec_state)
 
+        # Remove -1 from timestep
+        if hyp.timestep is not None and len(hyp.timestep) > 0 and hyp.timestep[0] == -1:
+            hyp.timestep = hyp.timestep[1:]
+
     return hypotheses
 
 
@@ -172,6 +183,13 @@ class BeamRNNTInfer(Typing):
 
             NOTE: `preserve_alignments` is an invalid argument for any `search_type`
             other than basic beam search.
+
+        ngram_lm_model: str
+            The path to the N-gram LM
+        ngram_lm_alpha: float
+            Alpha weight of N-gram LM
+        tokens_type: str
+            Tokenization type ['subword', 'char']
     """
 
     @property
@@ -209,6 +227,8 @@ class BeamRNNTInfer(Typing):
         language_model: Optional[Dict[str, Any]] = None,
         softmax_temperature: float = 1.0,
         preserve_alignments: bool = False,
+        ngram_lm_model: Optional[str] = None,
+        ngram_lm_alpha: float = 0.0,
     ):
         self.decoder = decoder_model
         self.joint = joint_model
@@ -266,10 +286,10 @@ class BeamRNNTInfer(Typing):
         self.maes_expansion_gamma = float(maes_expansion_gamma)
         self.maes_expansion_beta = int(maes_expansion_beta)
 
-        if self.maes_prefix_alpha < 0:
+        if self.search_type == 'maes' and self.maes_prefix_alpha < 0:
             raise ValueError("`maes_prefix_alpha` must be a positive integer.")
 
-        if self.vocab_size < beam_size + maes_expansion_beta:
+        if self.search_type == 'maes' and self.vocab_size < beam_size + maes_expansion_beta:
             raise ValueError(
                 f"beam_size ({beam_size}) + expansion_beta ({maes_expansion_beta}) "
                 f"should be smaller or equal to vocabulary size ({self.vocab_size})."
@@ -278,7 +298,7 @@ class BeamRNNTInfer(Typing):
         if search_type == 'maes':
             self.max_candidates += maes_expansion_beta
 
-        if self.maes_num_steps < 2:
+        if self.search_type == 'maes' and self.maes_num_steps < 2:
             raise ValueError("`maes_num_steps` must be greater than 1.")
 
         if softmax_temperature != 1.0 and language_model is not None:
@@ -292,6 +312,19 @@ class BeamRNNTInfer(Typing):
         self.language_model = language_model
         self.preserve_alignments = preserve_alignments
 
+        self.token_offset = 0
+
+        if ngram_lm_model:
+            if KENLM_AVAILABLE:
+                self.ngram_lm = kenlm.Model(ngram_lm_model)
+                self.ngram_lm_alpha = ngram_lm_alpha
+            else:
+                raise ImportError(
+                    "KenLM package (https://github.com/kpu/kenlm) is not installed. " "Use ngram_lm_model=None."
+                )
+        else:
+            self.ngram_lm = None
+
     @typecheck()
     def __call__(
         self,
@@ -302,7 +335,7 @@ class BeamRNNTInfer(Typing):
         """Perform general beam search.
 
         Args:
-            encoder_output: Encoded speech features (B, T_max, D_enc)
+            encoder_output: Encoded speech features (B, D_enc, T_max)
             encoded_lengths: Lengths of the encoder outputs
 
         Returns:
@@ -425,7 +458,8 @@ class BeamRNNTInfer(Typing):
             not_blank = True
             symbols_added = 0
 
-            while not_blank and (symbols_added < self.max_candidates):
+            # TODO: Figure out how to remove this hard coding afterwords
+            while not_blank and (symbols_added < 5):
                 ytu = torch.log_softmax(self.joint.joint(hi, y) / self.softmax_temperature, dim=-1)  # [1, 1, 1, V + 1]
                 ytu = ytu[0, 0, 0, :]  # [V + 1]
 
@@ -568,6 +602,7 @@ class BeamRNNTInfer(Typing):
 
                         hyps.append(new_hyp)
 
+                    # Determine whether the alignment should be blank or token
                     if self.preserve_alignments:
                         if k == self.blank:
                             new_hyp.alignments[-1].append(
@@ -619,9 +654,6 @@ class BeamRNNTInfer(Typing):
         Returns:
             nbest_hyps: N-best decoding results
         """
-        if self.preserve_alignments:
-            raise NotImplementedError("`preseve_alignments` is not implemented for Time-Synchronous Decoding.")
-
         if partial_hypotheses is not None:
             raise NotImplementedError("`partial_hypotheses` support is not supported")
 
@@ -653,6 +685,11 @@ class BeamRNNTInfer(Typing):
         ]
         cache = {}
 
+        # Initialize alignments
+        if self.preserve_alignments:
+            for hyp in B:
+                hyp.alignments = [[]]
+
         for i in range(int(encoded_lengths)):
             hi = h[:, i : i + 1, :]
 
@@ -683,16 +720,23 @@ class BeamRNNTInfer(Typing):
                     if hyp.y_sequence not in seq_A:
                         # If the sequence is not in seq_A, add it as the blank token
                         # In this step, we dont add a token but simply update score
-                        A.append(
-                            Hypothesis(
-                                score=(hyp.score + float(beam_logp[j, self.blank])),
-                                y_sequence=hyp.y_sequence[:],
-                                dec_state=hyp.dec_state,
-                                lm_state=hyp.lm_state,
-                                timestep=hyp.timestep[:],
-                                length=encoded_lengths,
-                            )
+                        _temp_hyp = Hypothesis(
+                            score=(hyp.score + float(beam_logp[j, self.blank])),
+                            y_sequence=hyp.y_sequence[:],
+                            dec_state=hyp.dec_state,
+                            lm_state=hyp.lm_state,
+                            timestep=hyp.timestep[:],
+                            length=encoded_lengths,
                         )
+
+                        # Preserve the blank token alignment
+                        if self.preserve_alignments:
+                            _temp_hyp.alignments = copy.deepcopy(hyp.alignments)
+                            _temp_hyp.alignments[-1].append(
+                                (beam_logp[j].clone(), torch.tensor(self.blank, dtype=torch.int32)),
+                            )
+
+                        A.append(_temp_hyp)
                     else:
                         # merge the existing blank hypothesis score with current score.
                         dict_pos = seq_A.index(hyp.y_sequence)
@@ -717,14 +761,45 @@ class BeamRNNTInfer(Typing):
                                 length=encoded_lengths,
                             )
 
+                            # Preserve token alignment
+                            if self.preserve_alignments:
+                                new_hyp.alignments = copy.deepcopy(hyp.alignments)
+                                new_hyp.alignments[-1].append(
+                                    (beam_topk[0].clone().cpu(), torch.tensor(k, dtype=torch.int32)),
+                                )
+
                             D.append(new_hyp)
 
                 # Prune beam
                 C = sorted(D, key=lambda x: x.score, reverse=True)[:beam]
 
+                if self.preserve_alignments:
+                    # convert Ti-th logits into a torch array
+                    for C_i in C:
+                        # Check if the last token emitted at last timestep was a blank
+                        # If so, move to next timestep
+                        logp, label = C_i.alignments[-1][-1]  # The last alignment of this step
+                        if int(label) == self.blank:
+                            C_i.alignments.append([])  # blank buffer for next timestep
+
             # Prune beam
             B = sorted(A, key=lambda x: x.score, reverse=True)[:beam]
 
+            if self.preserve_alignments:
+                # convert Ti-th logits into a torch array
+                for B_i in B:
+                    # Check if the last token emitted at last timestep was a blank
+                    # If so, move to next timestep
+                    logp, label = B_i.alignments[-1][-1]  # The last alignment of this step
+                    if int(label) == self.blank:
+                        B_i.alignments.append([])  # blank buffer for next timestep
+
+        # Remove trailing empty list of alignments
+        if self.preserve_alignments:
+            for h in B:
+                if len(h.alignments[-1]) == 0:
+                    del h.alignments[-1]
+
         return self.sort_nbest(B)
 
     def align_length_sync_decoding(
@@ -739,11 +814,6 @@ class BeamRNNTInfer(Typing):
         Returns:
             nbest_hyps: N-best decoding results
         """
-        if self.preserve_alignments:
-            raise NotImplementedError(
-                "`preseve_alignments` is not implemented for Alignment-length Synchronous Decoding."
-            )
-
         if partial_hypotheses is not None:
             raise NotImplementedError("`partial_hypotheses` support is not supported")
 
@@ -784,6 +854,10 @@ class BeamRNNTInfer(Typing):
             )
         ]
 
+        # Initialize alignments
+        if self.preserve_alignments:
+            B[0].alignments = [[]]
+
         final = []
         cache = {}
 
@@ -865,6 +939,14 @@ class BeamRNNTInfer(Typing):
                         length=i,
                     )
 
+                    if self.preserve_alignments:
+                        new_hyp.alignments = copy.deepcopy(hyp.alignments)
+
+                        # Add the alignment of blank at this step
+                        new_hyp.alignments[-1].append(
+                            (beam_logp[j].clone().cpu(), torch.tensor(self.blank, dtype=torch.int32))
+                        )
+
                     # Add blank prediction to A
                     A.append(new_hyp)
 
@@ -894,6 +976,14 @@ class BeamRNNTInfer(Typing):
                             length=i,
                         )
 
+                        if self.preserve_alignments:
+                            new_hyp.alignments = copy.deepcopy(hyp.alignments)
+
+                            # Add the alignment of Uj for this beam candidate at this step
+                            new_hyp.alignments[-1].append(
+                                (beam_logp[j].clone().cpu(), torch.tensor(new_hyp.y_sequence[-1], dtype=torch.int32))
+                            )
+
                         A.append(new_hyp)
 
                 # Prune and recombine same hypothesis
@@ -902,13 +992,35 @@ class BeamRNNTInfer(Typing):
                 B = sorted(A, key=lambda x: x.score, reverse=True)[:beam]
                 B = self.recombine_hypotheses(B)
 
+                if self.preserve_alignments:
+                    # convert Ti-th logits into a torch array
+                    for B_i in B:
+                        # Check if the last token emitted at last timestep was a blank
+                        # If so, move to next timestep
+                        logp, label = B_i.alignments[-1][-1]  # The last alignment of this step
+                        if int(label) == self.blank:
+                            B_i.alignments.append([])  # blank buffer for next timestep
+
             # If B_ is empty list, then we may be able to early exit
             elif len(batch_ids) == len(batch_removal_ids):
+                # break early
                 break
 
         if final:
+            # Remove trailing empty list of alignments
+            if self.preserve_alignments:
+                for h in final:
+                    if len(h.alignments[-1]) == 0:
+                        del h.alignments[-1]
+
             return self.sort_nbest(final)
         else:
+            # Remove trailing empty list of alignments
+            if self.preserve_alignments:
+                for h in B:
+                    if len(h.alignments[-1]) == 0:
+                        del h.alignments[-1]
+
             return B
 
     def modified_adaptive_expansion_search(
@@ -923,11 +1035,6 @@ class BeamRNNTInfer(Typing):
         Returns:
             nbest_hyps: N-best decoding results
         """
-        if self.preserve_alignments:
-            raise NotImplementedError(
-                "`preseve_alignments` is not implemented for Alignment-length Synchronous Decoding."
-            )
-
         if partial_hypotheses is not None:
             raise NotImplementedError("`partial_hypotheses` support is not supported")
 
@@ -952,10 +1059,20 @@ class BeamRNNTInfer(Typing):
 
         cache = {}
 
+        # Initialize alignment buffer
+        if self.preserve_alignments:
+            for hyp in init_tokens:
+                hyp.alignments = [[]]
+
         # Decode a batch of beam states and scores
         beam_dec_out, beam_state, beam_lm_tokens = self.decoder.batch_score_hypothesis(init_tokens, cache, beam_state)
         state = self.decoder.batch_select_state(beam_state, 0)
 
+        # Setup ngram LM:
+        if self.ngram_lm:
+            init_lm_state = kenlm.State()
+            self.ngram_lm.BeginSentenceWrite(init_lm_state)
+
         # TODO: Setup LM
         if self.language_model is not None:
             # beam_lm_states, beam_lm_scores = self.lm.buff_predict(
@@ -979,8 +1096,17 @@ class BeamRNNTInfer(Typing):
                 dec_out=[beam_dec_out[0]],
                 lm_state=lm_state,
                 lm_scores=lm_scores,
+                timestep=[-1],
+                length=0,
             )
         ]
+        if self.ngram_lm:
+            kept_hyps[0].ngram_lm_state = init_lm_state
+
+        # Initialize alignment buffer
+        if self.preserve_alignments:
+            for hyp in kept_hyps:
+                hyp.alignments = [[]]
 
         for t in range(encoded_lengths):
             enc_out_t = h[t : t + 1].unsqueeze(0)  # [1, 1, D]
@@ -1029,7 +1155,11 @@ class BeamRNNTInfer(Typing):
                             dec_state=hyp.dec_state,
                             lm_state=hyp.lm_state,
                             lm_scores=hyp.lm_scores,
+                            timestep=hyp.timestep[:],
+                            length=t,
                         )
+                        if self.ngram_lm:
+                            new_hyp.ngram_lm_state = hyp.ngram_lm_state
 
                         # If the expansion was for blank
                         if k == self.blank:
@@ -1039,6 +1169,14 @@ class BeamRNNTInfer(Typing):
                             # new_hyp.y_sequence.append(int(k))
                             if (new_hyp.y_sequence + [int(k)]) not in duplication_check:
                                 new_hyp.y_sequence.append(int(k))
+                                new_hyp.timestep.append(t)
+
+                                # Setup ngram LM:
+                                if self.ngram_lm:
+                                    lm_score, new_hyp.ngram_lm_state = self.compute_ngram_score(
+                                        hyp.ngram_lm_state, int(k)
+                                    )
+                                    new_hyp.score += self.ngram_lm_alpha * lm_score
 
                                 # TODO: Setup LM
                                 if self.language_model is not None:
@@ -1049,11 +1187,38 @@ class BeamRNNTInfer(Typing):
 
                                 list_exp.append(new_hyp)
 
+                        # Preserve alignments
+                        if self.preserve_alignments:
+                            new_hyp.alignments = copy.deepcopy(hyp.alignments)
+
+                            if k == self.blank:
+                                new_hyp.alignments[-1].append(
+                                    (beam_logp[i].clone().cpu(), torch.tensor(self.blank, dtype=torch.int32)),
+                                )
+                            else:
+                                new_hyp.alignments[-1].append(
+                                    (
+                                        beam_logp[i].clone().cpu(),
+                                        torch.tensor(new_hyp.y_sequence[-1], dtype=torch.int32),
+                                    ),
+                                )
+
                 # If there were no token expansions in any of the hypotheses,
                 # Early exit
                 if not list_exp:
                     kept_hyps = sorted(list_b, key=lambda x: x.score, reverse=True)[:beam]
 
+                    # Update aligments with next step
+                    if self.preserve_alignments:
+                        # convert Ti-th logits into a torch array
+                        for h_i in kept_hyps:
+                            # Check if the last token emitted at last timestep was a blank
+                            # If so, move to next timestep
+                            logp, label = h_i.alignments[-1][-1]  # The last alignment of this step
+                            if int(label) == self.blank:
+                                h_i.alignments.append([])  # blank buffer for next timestep
+
+                    # Early exit
                     break
 
                 else:
@@ -1102,6 +1267,17 @@ class BeamRNNTInfer(Typing):
 
                         # Copy the expanded hypothesis
                         hyps = list_exp[:]
+
+                        # Update aligments with next step
+                        if self.preserve_alignments:
+                            # convert Ti-th logits into a torch array
+                            for h_i in hyps:
+                                # Check if the last token emitted at last timestep was a blank
+                                # If so, move to next timestep
+                                logp, label = h_i.alignments[-1][-1]  # The last alignment of this step
+                                if int(label) == self.blank:
+                                    h_i.alignments.append([])  # blank buffer for next timestep
+
                     else:
                         # Extract the log probabilities
                         beam_logp = torch.log_softmax(
@@ -1128,6 +1304,22 @@ class BeamRNNTInfer(Typing):
                         # Finally, update the kept hypothesis of sorted top Beam candidates
                         kept_hyps = sorted(list_b + list_exp, key=lambda x: x.score, reverse=True)[:beam]
 
+                        # Update aligments with next step
+                        if self.preserve_alignments:
+                            # convert Ti-th logits into a torch array
+                            for h_i in kept_hyps:
+                                # Check if the last token emitted at last timestep was a blank
+                                # If so, move to next timestep
+                                logp, label = h_i.alignments[-1][-1]  # The last alignment of this step
+                                if int(label) == self.blank:
+                                    h_i.alignments.append([])  # blank buffer for next timestep
+
+        # Remove trailing empty list of alignments
+        if self.preserve_alignments:
+            for h in kept_hyps:
+                if len(h.alignments[-1]) == 0:
+                    del h.alignments[-1]
+
         # Sort the hypothesis with best scores
         return self.sort_nbest(kept_hyps)
 
@@ -1172,21 +1364,53 @@ class BeamRNNTInfer(Typing):
                         self.joint.joint(enc_out, hyp_i.dec_out[-1]) / self.softmax_temperature, dim=-1,
                     )
                     logp = logp[0, 0, 0, :]
-
                     curr_score = hyp_i.score + float(logp[hyp_j.y_sequence[pref_id]])
+                    # Setup ngram LM:
+                    if self.ngram_lm:
+                        lm_score, next_state = self.compute_ngram_score(
+                            hyp_i.ngram_lm_state, int(hyp_j.y_sequence[pref_id])
+                        )
+                        curr_score += self.ngram_lm_alpha * lm_score
 
                     for k in range(pref_id, (curr_id - 1)):
                         logp = torch.log_softmax(
                             self.joint.joint(enc_out, hyp_j.dec_out[k]) / self.softmax_temperature, dim=-1,
                         )
                         logp = logp[0, 0, 0, :]
-
                         curr_score += float(logp[hyp_j.y_sequence[k + 1]])
+                        # Setup ngram LM:
+                        if self.ngram_lm:
+                            lm_score, next_state = self.compute_ngram_score(next_state, int(hyp_j.y_sequence[k + 1]))
+                            curr_score += self.ngram_lm_alpha * lm_score
 
                     hyp_j.score = np.logaddexp(hyp_j.score, curr_score)
 
         return hypotheses
 
+    def compute_ngram_score(self, current_lm_state: "kenlm.State", label: int) -> Tuple[float, "kenlm.State"]:
+        """
+        Score computation for kenlm ngram language model.
+        """
+
+        if self.token_offset:
+            label = chr(label + self.token_offset)
+        else:
+            label = str(label)
+        next_state = kenlm.State()
+        lm_score = self.ngram_lm.BaseScore(current_lm_state, label, next_state)
+        lm_score *= 1.0 / np.log10(np.e)
+
+        return lm_score, next_state
+
+    def set_decoding_type(self, decoding_type: str):
+
+        # Please check train_kenlm.py in scripts/asr_language_modeling/ to find out why we need
+        # TOKEN_OFFSET for BPE-based models
+        if decoding_type == 'subword':
+            from nemo.collections.asr.parts.submodules.ctc_beam_decoding import DEFAULT_TOKEN_OFFSET
+
+            self.token_offset = DEFAULT_TOKEN_OFFSET
+
 
 @dataclass
 class BeamRNNTInferConfig:
@@ -1205,3 +1429,5 @@ class BeamRNNTInferConfig:
     language_model: Optional[Dict[str, Any]] = None
     softmax_temperature: float = 1.0
     preserve_alignments: bool = False
+    ngram_lm_model: Optional[str] = None
+    ngram_lm_alpha: Optional[float] = 0.0
