@@ -13,6 +13,12 @@
 # limitations under the License.
 
 # from nemo.collections.nlp.models.language_modeling.megatron.bert_model import BertModel
-from nemo.collections.nlp.models.language_modeling.megatron.gpt_model import GPTModel
+
+try:
+    from nemo.collections.nlp.models.language_modeling.megatron.gpt_model import GPTModel
+
+    HAVE_APEX = True
+except (ImportError, ModuleNotFoundError):
+    HAVE_APEX = False
 
 # from nemo.collections.nlp.models.language_modeling.megatron.t5_model import T5Model
