@@ -25,9 +25,7 @@ from nemo.collections.common.tokenizers.sentencepiece_tokenizer import SentenceP
 from nemo.collections.nlp.models.language_modeling.megatron_base_model import MegatronBaseModel
 from nemo.collections.nlp.modules.common import (
     PromptEncoder,
-    PromptEncoderMLP,
     PromptEncoderType,
-    PromptTable,
     VirtualPromptPlaceholderToken,
     VirtualPromptSource,
     VirtualPromptStyle,
@@ -60,11 +58,13 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
 
     def __init__(self, cfg: DictConfig, trainer: Trainer):
         super().__init__(cfg, trainer)
+        self.init_model(cfg, trainer)
 
+    def init_model(self, cfg: DictConfig, trainer: Trainer):
         self.cfg = cfg
 
         self.load_frozen_model(cfg, trainer)
-
+        self.prompt_encoder = None
         self.tokenizer = self.frozen_model.tokenizer
 
         if hasattr(self.frozen_model.cfg, "encoder") and hasattr(self.frozen_model.cfg, "decoder"):
@@ -83,37 +83,18 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
 
         if self.first_stage_of_pipeline() and self.virtual_prompt_style in [
             VirtualPromptStyle.P_TUNING,
-            VirtualPromptStyle.PROMPT_TUNING,
-            VirtualPromptStyle.INFERENCE,
         ]:
 
             # TODO: Handle this when moving GPT prompt learning to the base class.
             self.word_embeddings = self.frozen_model.enc_dec_model.encoder_embedding.word_embeddings
 
-            # Prompt table stores all task embeddings, p-tuning virtual prompts get added to the table after training
-            self.prompt_table = PromptTable(
-                existing_tasks=self.existing_tasks,
-                task_templates=self.task_templates,
-                task_id_num_to_name=self.task_id_num_to_name,
-                hidden_size=self.hidden_size,
-            )
-
-        self._prompt_table_key = VirtualPromptSource.PROMPT_TABLE.value
-        self._prompt_encoder_key = VirtualPromptSource.PROMPT_ENCODER.value
-
-        # Prompt tuning stores virtual prompts in the prompt table and tunes their weight directly
-        if self.virtual_prompt_style in [VirtualPromptStyle.PROMPT_TUNING, VirtualPromptStyle.INFERENCE]:
-            self.virtual_prompt_source = VirtualPromptSource.PROMPT_TABLE
-
         # P-Tuning uses an LSTM Encoder to produce virtual token embeddings
-        elif self.virtual_prompt_style == VirtualPromptStyle.P_TUNING:
+        if self.virtual_prompt_style == VirtualPromptStyle.P_TUNING:
             self.virtual_prompt_source = VirtualPromptSource.PROMPT_ENCODER
         elif self.virtual_prompt_style == VirtualPromptStyle.NO_PROMPT:
             self.virtual_prompt_source = VirtualPromptSource.NO_PROMPT
         else:
-            raise ValueError(
-                f"\nvirtual prompt style '{cfg.virtual_prompt_style}' not recognized, please use one of 'prompt-tuning' or 'p-tuning'"
-            )
+            raise ValueError(f"\nvirtual prompt style '{cfg.virtual_prompt_style}'")
 
         self._reduced_loss_buffer = []
         self._inference_config = None
@@ -140,6 +121,7 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
         # make sure the default pytorch lightning gradient clipping in the basemodel
         self.grad_clip_pl_default = True
         self.lowest_val_loss = None
+        self.prompt_encoder = None
 
     def load_task_templates(self, task_templates):
         """
@@ -180,30 +162,6 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
                 for taskname in self.new_tasks
             ), "Total virtual tokens for each task tuned simultaneously must match. If you want to use a different number of virtual tokens for different tasks, tune them separately."
 
-    def init_new_prompts(self):
-        """
-        Initialize new virtual prompts to be tuned using prompt tuning 
-        """
-        for idx, taskname in enumerate(self.new_tasks):
-            init_method = self.cfg.prompt_tuning.new_prompt_init_methods[idx].lower()
-            total_virtual_tokens = self.task_templates[taskname]["total_virtual_tokens"]
-
-            if init_method == "text":
-                init_text = self.cfg.prompt_tuning.new_prompt_init_text[idx]
-                init_text_ids = self.tokenizer.text_to_ids(init_text)
-                self.prompt_table.init_prompt_from_text(
-                    taskname, init_text_ids, self.word_embeddings, total_virtual_tokens
-                )
-
-            elif init_method == 'random':
-                self.prompt_table.init_prompt_from_random(taskname, total_virtual_tokens)
-
-            else:
-                raise AttributeError(
-                    f'\nvirtual prompt init method {init_method} is not recognized\
-                                        please use one of text or random'
-                )
-
     def init_prompt_encoder(self):
         """
         Init the prompt encoder needed for p-tuning on a new task
@@ -212,75 +170,25 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
         new_task = self.new_tasks[0]
         total_virtual_tokens = self.task_templates[new_task]["total_virtual_tokens"]
 
-        encoder_type = PromptEncoderType(self.cfg.p_tuning.get("encoder_type", "mlp").lower())
-        self.prompt_encoder = None
-        if encoder_type == PromptEncoderType.TPMLP:
-            self.prompt_encoder = PromptEncoderMLP(
-                total_virtual_tokens=total_virtual_tokens,
-                hidden_size=self.cfg.p_tuning.get("encoder_hidden", 2048),
-                output_size=self.hidden_size,
-                init_std=self.cfg.p_tuning.get("init_std", 0.023),
-            )
-        elif encoder_type == PromptEncoderType.LSTM or encoder_type == PromptEncoderType.MLP:
-            self.prompt_encoder = PromptEncoder(
-                encoder_type=PromptEncoderType(encoder_type),
-                total_virtual_tokens=total_virtual_tokens,
-                token_dim=self.hidden_size,
-                hidden_size=self.cfg.p_tuning.get("encoder_hidden", self.hidden_size // 2),
-                lstm_dropout=self.cfg.p_tuning.get("dropout", 0.0),
-                num_layers=self.cfg.p_tuning.get("num_layers", 2),
-            )
-        else:
-            raise ValueError("Prompt encoder type not recognized. Please use one of MLP (recommended) or LSTM.")
+        encoder_type = PromptEncoderType(self.cfg.p_tuning.get("encoder_type", "tpmlp").lower())
+        self.prompt_encoder = PromptEncoder(
+            encoder_type=encoder_type,
+            total_virtual_tokens=total_virtual_tokens,
+            token_dim=self.hidden_size,
+            hidden_size=self.cfg.p_tuning.get("encoder_hidden", self.hidden_size // 2),
+            lstm_dropout=self.cfg.p_tuning.get("dropout", 0.0),
+            num_layers=self.cfg.p_tuning.get("num_layers", 2),
+            init_std=self.cfg.p_tuning.get("init_std", 0.023),
+            taskname=new_task,
+        )
 
-    def add_ptuned_prompts_to_prompt_table(self):
-        """
-        Adds all newly p-tuned virtual prompts to the prompt table 
-        for inference. p-tuned virtual prompts WILL NOT be further
-        tuned once added to the prompt table. After p-tuned prompts
-        are added to the prompt table, the prompt encoder weights
-        are removed from the model to avoid needing to load unnecessary
-        weights during inference or future p-tuning/prompt-tuning.
-        """
-        # Save p-tuned prompts to prompt table
-        if self.virtual_prompt_style == VirtualPromptStyle.P_TUNING and self.first_stage_of_pipeline():
-            for taskname in self.new_tasks:
-                device = next(self.word_embeddings.parameters()).device
-                tokenized_taskname = torch.tensor(self.tokenizer.text_to_ids(taskname)).to(device)
-                taskname_embeddings = self.word_embeddings(tokenized_taskname).unsqueeze(0)
-                virtual_prompt_embeddings = self.prompt_encoder(taskname_embeddings=taskname_embeddings).squeeze(0)
-                total_virtual_tokens = self.task_templates[taskname]["total_virtual_tokens"]
-                self.prompt_table.add_prompt_from_p_tuning_encoder(
-                    taskname, virtual_prompt_embeddings, total_virtual_tokens
-                )
-
-    def freeze_existing_virtual_prompt_params(self):
+    def freeze_existing_word_embeddings(self):
         """Freeze params of existing virtual prompts that should not be tuned further
         """
-        # Only want new prompt tags to be tunable, leave existing prompt tags alone
-        for taskname in self.prompt_table.prompt_table.keys():
-            if taskname in set(self.new_tasks):
-                for params in self.prompt_table.prompt_table[taskname].parameters():
-                    params.requires_grad = True
-            else:
-                for params in self.prompt_table.prompt_table[taskname].parameters():
-                    params.requires_grad = False
-
         # Make sure word embeddings are frozen
         for params in self.word_embeddings.parameters():
             params.requires_grad = False
 
-    def get_model_tasks(self):
-        """
-        For user to inspect which tasks the model has been
-        p-tuned/prompt-tuned to preform.
-        """
-        tasks = {}
-        for taskname in self.prompt_table.prompt_table.keys():
-            tasks[taskname] = self.task_templates[taskname].copy()
-
-        return tasks
-
     def state_dict(self):
         """
         Custom state dict that only contains prompt table and prompt encoder parameters. 
@@ -291,10 +199,10 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
         state_dict_ = {}
 
         if self.first_stage_of_pipeline():
-            state_dict_[self._prompt_table_key] = self.prompt_table.state_dict()
-
             if self.virtual_prompt_source == VirtualPromptSource.PROMPT_ENCODER:
-                state_dict_[self._prompt_encoder_key] = self.prompt_encoder.state_dict()
+                state_dict_ = self.prompt_encoder.state_dict()
+            else:
+                raise ValueError("invalid virtual prompt source")
 
         return state_dict_
 
@@ -304,26 +212,33 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
         parameters. Matching load method for this class' custom state dict method. 
         """
         if self.first_stage_of_pipeline():
-            if self._prompt_table_key in state_dict:
-                state_dict_ = state_dict[self._prompt_table_key]
+            if self.virtual_prompt_source == VirtualPromptSource.PROMPT_ENCODER:
+                if self.prompt_encoder is None:
+                    self.init_prompt_encoder()
+                self.prompt_encoder.load_state_dict(state_dict, strict)
             else:
-                # Handle loading state dict before weight saving change for backward compatibility.
-                state_dict_ = OrderedDict()
-                for key in state_dict.keys():
-                    if key.startswith(self._prompt_table_key):
-                        key_substring = ".".join(key.split(".")[1:])
-                        state_dict_[key_substring] = state_dict[key]
+                raise ValueError("invalid virtual prompt source")
 
-            self.prompt_table.load_state_dict(state_dict_, strict)
+    def setup_optimizer_param_groups(self):
+        """
+        ModelPT override. Optimizer will get self._optimizer_param_groups. 
+        Only want virtual prompt params to be passed to the optimizer.
+        """
+        ## Freeze frozen model
+        for param in self.frozen_model.parameters():
+            param.requires_grad = False
 
-            if (
-                self._prompt_encoder_key in state_dict
-                and self.virtual_prompt_source == VirtualPromptSource.PROMPT_ENCODER
-            ):
-                state_dict_ = state_dict[self._prompt_encoder_key]
-                self.prompt_encoder.load_state_dict(state_dict_, strict)
+        virtual_prompt_params = {'params': []}
 
-    def embed_input_train(self, input_ids: Tensor, taskname_ids: Tensor):
+        if self.first_stage_of_pipeline():
+            if self.virtual_prompt_source == VirtualPromptSource.PROMPT_ENCODER:
+                virtual_prompt_params['params'].extend([param for param in self.prompt_encoder.parameters()])
+            else:
+                raise ValueError("Optimizer only supports Prompt Encoder.")
+
+        self._optimizer_param_groups = (virtual_prompt_params,)
+
+    def embed_input(self, input_ids: Tensor, taskname_ids: Tensor, use_cached_reps: bool):
         """
         Replaces the virtual tokens in the input_ids with embeddings 
         calculated from either the 'prompt_table' or 'prompt_encoder'. 
@@ -348,14 +263,12 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
         if not virtual_token_locations.any():
             return discrete_token_embeds
 
-        # Get virtual token embeddings from the prompt table or prompt encoder
-        if self.virtual_prompt_source == VirtualPromptSource.PROMPT_TABLE:
-            virtual_token_embeds = [self.prompt_table(task_id_num) for task_id_num in taskname_ids]
-            virtual_token_embeds = torch.stack(virtual_token_embeds)
-
-        elif self.virtual_prompt_source == VirtualPromptSource.PROMPT_ENCODER:
-            taskname_embeddings = self.word_embeddings(taskname_ids)
-            virtual_token_embeds = self.prompt_encoder(taskname_embeddings=taskname_embeddings)
+        if self.virtual_prompt_source == VirtualPromptSource.PROMPT_ENCODER:
+            # taskname_embeddings = self.word_embeddings(taskname_ids)
+            batch_size, _ = taskname_ids.size()
+            virtual_token_embeds = self.prompt_encoder(batch_size=batch_size, use_cached_reps=use_cached_reps)
+        else:
+            raise ValueError("invalid VirtualPromptSource.")
 
         # Create index template specifying where virtual token embeddings should be placed
         batch_size, _, embedding_size = discrete_token_embeds.shape
@@ -373,110 +286,13 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
 
         return input_embeds
 
-    def embed_input_inference(self, input_ids: Tensor, taskname_ids: Tensor):
-        """
-        Replaces the virtual tokens in the input_ids with embeddings the 
-        'prompt_table' only. The virtual token placeholders each have their
-        own unique token_id within `self.pseudo_token_ids` to facilitate 
-        placing the virtual tokens in their correct locations at each 
-        decoding time step. 
-
-        params:
-            input_ids: the input token ids
-            taskname_ids: the NLP task tag token ids
-        returns:
-            the token embedding for the LM model.
-        """
-        batch_size, seq_length = input_ids.shape
-
-        # Replace virtual token ids with padding for forward pass through vocab embeddings
-        discrete_token_ids = input_ids.clone()
-        discrete_token_ids[(input_ids >= self.pseudo_token_ids_start)] = self.pad_token_id
-        discrete_token_embeds = self.word_embeddings(discrete_token_ids).clone()
-
-        # Find the indicies where virtual tokens should be inserted
-        virtual_token_locations = input_ids >= self.pseudo_token_ids_start
-        virtual_token_locations = virtual_token_locations.unsqueeze(-1)
-        virtual_token_locations = virtual_token_locations.expand(batch_size, seq_length, self.hidden_size)
-
-        # If there are no virtual tokens, just return discrete token embeds
-        if not virtual_token_locations.any():
-            return discrete_token_embeds
-
-        # Convert virtual token vocab_id to virtual token embedding idx
-        virtual_token_ids = input_ids.clone()
-        virtual_token_ids = torch.sub(virtual_token_ids, self.pseudo_token_ids_start)
-        virtual_token_ids = torch.clamp(virtual_token_ids, min=0)
-
-        # Only get needed virtual token embeddings from the prompt table according to virtual token ids
-        virtual_token_embeds = [self.prompt_table(taskname_ids[i], virtual_token_ids[i]) for i in range(batch_size)]
-        virtual_token_embeds = torch.stack(virtual_token_embeds)
-
-        # Make sure discrete_token_embeds and virtual_token_embeds share the same dtype
-        discrete_token_embeds = discrete_token_embeds.type(virtual_token_embeds.dtype)
-
-        # Put virtual and discrete token embs in their correct locations for final output
-        input_embeds = torch.where(virtual_token_locations, virtual_token_embeds, discrete_token_embeds)
-        return input_embeds
-
-    def update_config_for_inference_and_save(self):
-        self.virtual_prompt_style = VirtualPromptStyle.INFERENCE
-        self.virtual_prompt_source = VirtualPromptSource.PROMPT_TABLE
-
-        # Move new tags to existing tag list for loading during inference later
-        with open_dict(self.cfg):
-            self.cfg.existing_tasks = self.existing_tasks + self.new_tasks
-            self.cfg.new_tasks = []
-            self.cfg.virtual_prompt_style = VirtualPromptStyle.INFERENCE.value
-
-        # Save the best nemo model
-        self.save_to(save_path=self.cfg.nemo_path)
-        logging.info(f"An inference ready model was saved to {self.cfg.nemo_path}")
-
-    def save_checkpoint_as_nemo_file(self):
-        self.add_ptuned_prompts_to_prompt_table()
-
-        # Save current config and state dict params in temp values
-        current_virtual_prompt_style = self.virtual_prompt_style
-        current_virtual_prompt_source = self.virtual_prompt_source
-        current_existing_tasks = self.cfg.existing_tasks
-        current_new_tasks = self.cfg.new_tasks
-
-        # Temporarily overwrite params to save an inference ready .nemo checkpoint
-        self.update_config_for_inference_and_save()
-
-        # Set values back to their training state to continue training
-        self.virtual_prompt_style = current_virtual_prompt_style
-        self.virtual_prompt_source = current_virtual_prompt_source
-
-        # Revert prompt table back to previous state
-        if self.virtual_prompt_style == VirtualPromptStyle.P_TUNING and self.first_stage_of_pipeline():
-            for taskname in current_new_tasks:
-                if taskname in self.prompt_table.prompt_table:
-                    del self.prompt_table.prompt_table[taskname]
-
-        with open_dict(self.cfg):
-            self.cfg.existing_tasks = current_existing_tasks
-            self.cfg.new_tasks = current_new_tasks
-            self.cfg.virtual_prompt_style = current_virtual_prompt_style.value
-
     def on_train_end(self):
         # Save p-tuned prompts to prompt table for inference or future task training
-        if self.virtual_prompt_style == VirtualPromptStyle.P_TUNING:
-            self.add_ptuned_prompts_to_prompt_table()
-            logging.info(f"All p-tuned prompts where moved to the prompt table.")
-
-            # Remove prompt encoder from model
-            self.prompt_encoder = None
-            logging.info(f"Prompt encoder deleted")
-
-        self.update_config_for_inference_and_save()
+        self.save_to(save_path=self.cfg.nemo_path)
 
     def setup(self, stage=None):
-        if (
-            stage == 'predict' or self.virtual_prompt_style == VirtualPromptStyle.INFERENCE
-        ) and self.first_stage_of_pipeline():
-            self.freeze_existing_virtual_prompt_params()
+        if stage == 'predict' and self.first_stage_of_pipeline():
+            self.freeze_existing_word_embeddings()
             return
 
         self.setup_test_data()
@@ -484,11 +300,9 @@ class MegatronBasePromptLearningModel(MegatronBaseModel, TextGeneration):
             return
 
         if self.first_stage_of_pipeline():
-            if self.virtual_prompt_style == VirtualPromptStyle.PROMPT_TUNING:
-                self.init_new_prompts()
-            elif self.virtual_prompt_style == VirtualPromptStyle.P_TUNING:
+            if self.virtual_prompt_style == VirtualPromptStyle.P_TUNING:
                 self.init_prompt_encoder()
-            self.freeze_existing_virtual_prompt_params()
+            self.freeze_existing_word_embeddings()
 
         self.setup_training_data()
         self.setup_validation_data()
