@@ -18,12 +18,13 @@ from typing import Any, Dict, List, Optional, Union
 import torch
 import torch.nn.functional as F
 from omegaconf.dictconfig import DictConfig
+from pytorch_lightning.plugins.precision.native_amp import NativeMixedPrecisionPlugin
 from pytorch_lightning.trainer.trainer import Trainer
 
 from nemo.collections.nlp.data.language_modeling.megatron import dataset_utils
-from nemo.collections.nlp.data.language_modeling.megatron.megatron_batch_samplers import (
-    MegatronPretrainingBatchSampler,
-    MegatronPretrainingRandomBatchSampler,
+from nemo.collections.nlp.data.language_modeling.megatron.data_samplers import (
+    MegatronPretrainingRandomSampler,
+    MegatronPretrainingSampler,
 )
 from nemo.collections.nlp.models.language_modeling.megatron.bert_model import BertModel
 from nemo.collections.nlp.models.language_modeling.megatron_base_model import MegatronBaseModel
@@ -32,6 +33,7 @@ from nemo.collections.nlp.modules.common.megatron.utils import (
     average_losses_across_data_parallel_group,
     get_params_for_weight_decay_optimization,
 )
+from nemo.collections.nlp.parts.nlp_overrides import GradScaler
 from nemo.collections.nlp.parts.utils_funcs import get_last_rank
 from nemo.core.classes.common import PretrainedModelInfo
 from nemo.core.neural_types import ChannelType, MaskType, NeuralType
@@ -118,6 +120,13 @@ class MegatronBertModel(MegatronBaseModel):
             else:
                 self.model = Float16Module(module=self.model, precision=cfg.precision)
 
+        if hasattr(self, '_nsys_profile_enabled'):
+            mp_size = cfg.get('tensor_model_parallel_size', 1) * cfg.get('pipeline_model_parallel_size', 1)
+            data_parallel_world_size = trainer.world_size // mp_size
+            grad_accum_steps = cfg.get('global_batch_size') // (cfg.get('micro_batch_size') * data_parallel_world_size)
+            self._nsys_profile_start_step *= grad_accum_steps
+            self._nsys_profile_end_step *= grad_accum_steps
+
     def model_provider_func(self, pre_process, post_process):
         cfg = self.cfg
         num_tokentypes = 2 if cfg.bert_binary_head else 0
@@ -179,26 +188,34 @@ class MegatronBertModel(MegatronBaseModel):
         return fwd_bwd_function
 
     def get_forward_output_and_loss_func(self):
-        def fwd_output_and_loss_func(batch, model, checkpoint_activations_all_layers=None):
+        def fwd_output_and_loss_func(dataloader_iter, model, checkpoint_activations_all_layers=None):
             if parallel_state.get_pipeline_model_parallel_world_size() == 1:
-                batch = [x.cuda(non_blocking=True) for x in batch]
-                tokens, types, sentence_order, loss_mask, lm_labels, padding_mask = batch
+                batch = next(dataloader_iter)
+                tokens, types, sentence_order, loss_mask, lm_labels, padding_mask = (
+                    batch['text'].cuda(non_blocking=True),
+                    batch['types'].cuda(non_blocking=True),
+                    batch['is_random'].cuda(non_blocking=True),
+                    batch['loss_mask'].cuda(non_blocking=True),
+                    batch['labels'].cuda(non_blocking=True),
+                    batch['padding_mask'].cuda(non_blocking=True),
+                )
             else:
+                batch = next(dataloader_iter)
                 if parallel_state.is_pipeline_first_stage():
-                    tokens = batch[0].cuda(non_blocking=True)
-                    types = batch[1].cuda(non_blocking=True)
-                    sentence_order = batch[2].cuda(non_blocking=True)
-                    padding_mask = batch[5].cuda(non_blocking=True)
+                    tokens = batch['text'].cuda(non_blocking=True)
+                    types = batch['types'].cuda(non_blocking=True)
+                    sentence_order = batch['is_random'].cuda(non_blocking=True)
+                    padding_mask = batch['padding_mask'].cuda(non_blocking=True)
                     loss_mask, lm_labels = None, None
                 elif parallel_state.is_pipeline_last_stage():
-                    loss_mask = batch[3].cuda(non_blocking=True)
-                    lm_labels = batch[4].cuda(non_blocking=True)
-                    sentence_order = batch[2].cuda(non_blocking=True)
-                    padding_mask = batch[5].cuda(non_blocking=True)
+                    loss_mask = batch['loss_mask'].cuda(non_blocking=True)
+                    lm_labels = batch['labels'].cuda(non_blocking=True)
+                    sentence_order = batch['is_random'].cuda(non_blocking=True)
+                    padding_mask = batch['padding_mask'].cuda(non_blocking=True)
                     tokens, types = None, None
                 else:
-                    padding_mask = batch[5].cuda(non_blocking=True)
-                    sentence_order = batch[2].cuda(non_blocking=True)
+                    padding_mask = batch['padding_mask'].cuda(non_blocking=True)
+                    sentence_order = batch['is_random'].cuda(non_blocking=True)
                     tokens, types, loss_mask, lm_labels = None, None, None, None
 
             if not self.cfg.bert_binary_head:
@@ -262,15 +279,34 @@ class MegatronBertModel(MegatronBaseModel):
 
         return output_tensor
 
-    def training_step(self, batch, batch_idx):
+    def training_step(self, dataloader_iter, batch_idx):
 
         self._optimizer.zero_grad()
-        batch_for_pipeline = self.process_batch(batch)
+
+        if self.with_distributed_adam:
+            # hack to enable overlapping param sync and forward compute
+            # note: the distributed optimizer monkey-patches each
+            # parameter's __getattribute__ function so that it can
+            # launch parameter all-gathers the first time the
+            # parameter is accessed after the optimizer step. However,
+            # PyTorch directly passes embedding parameters into a C++,
+            # bypassing this process. A quick-and-dirty hack is to
+            # manually interact with the parameter.
+            modules = self.model if isinstance(self.model, list) else [self.model]
+            for module in modules:
+                if isinstance(module, Float16Module):
+                    module = module.module
+                module = module.language_model
+                if hasattr(module, 'embedding'):
+                    for param in module.embedding.parameters():
+                        param.data_ptr()
+
         tensor_shape = [self.cfg.encoder_seq_length, self.cfg.micro_batch_size, self.cfg.hidden_size]
 
         # handle asynchronous grad reduction
         custom_sync_context_handler = None
         custom_grad_sync_func = None
+        custom_param_sync_func = None
         if self.with_distributed_adam:
             if self.megatron_amp_o2:
                 # copy grads to main grad
@@ -279,6 +315,7 @@ class MegatronBertModel(MegatronBaseModel):
                 # keep grad tensors around
                 custom_sync_context_handler = lambda: self._optimizer.no_sync(greedy_grad_copy=False)
             custom_grad_sync_func = self.reduce_overlap_gradients
+            custom_param_sync_func = self.sync_overlap_parameters
         else:
             if self.megatron_amp_o2 and not self.cfg.get('sequence_parallel', False):
                 custom_sync_context_handler = self._optimizer.no_sync
@@ -292,7 +329,7 @@ class MegatronBertModel(MegatronBaseModel):
 
         losses_reduced_per_micro_batch = fwd_bwd_function(
             forward_step_func=self.get_forward_output_and_loss_func(),
-            batch=batch_for_pipeline,
+            batch=dataloader_iter,
             model=self.model,
             forward_only=False,
             tensor_shape=tensor_shape,
@@ -300,6 +337,7 @@ class MegatronBertModel(MegatronBaseModel):
             grad_scaler=self.trainer.precision_plugin.scaler if self.cfg.precision == 16 else None,
             custom_sync_context_handler=custom_sync_context_handler,
             custom_grad_sync_func=custom_grad_sync_func,
+            custom_param_sync_func=custom_param_sync_func,
             sequence_parallel_enabled=self.cfg.get('sequence_parallel', False),
             num_micro_batches_with_partial_activation_checkpoints=self.cfg.get(
                 'num_micro_batches_with_partial_activation_checkpoints', None
@@ -340,21 +378,22 @@ class MegatronBertModel(MegatronBaseModel):
         if self.cfg.precision == 16:
             loss_scale = self.trainer.precision_plugin.scaler._scale
             if loss_scale is not None:
-                self.log('loss_scale', loss_scale)
+                self.log('loss_scale', loss_scale, batch_size=1)
 
         if (batch_idx + 1) % self.trainer.accumulate_grad_batches == 0:
             # Reduced loss for logging.
-            self.log('reduced_train_loss', loss_mean[0], prog_bar=True)
+            self.log('reduced_train_loss', loss_mean[0], prog_bar=True, batch_size=1)
             if len(loss_mean) > 2:
-                self.log('reduced_lm_train_loss', loss_mean[1], prog_bar=True)
-                self.log('reduced_sop_train_loss', loss_mean[2], prog_bar=True)
+                self.log('reduced_lm_train_loss', loss_mean[1], prog_bar=True, batch_size=1)
+                self.log('reduced_sop_train_loss', loss_mean[2], prog_bar=True, batch_size=1)
             lr = self._optimizer.param_groups[0]['lr']
-            self.log('lr', lr)
-            self.log('global_step', self.trainer.global_step, prog_bar=True)
+            self.log('lr', lr, batch_size=1)
+            self.log('global_step', self.trainer.global_step, prog_bar=True, batch_size=1)
             self.log(
                 'consumed_samples',
                 self.compute_consumed_samples(self.trainer.global_step - self.init_global_step),
                 prog_bar=True,
+                batch_size=1,
             )
 
         return loss_mean[0]
@@ -389,15 +428,14 @@ class MegatronBertModel(MegatronBaseModel):
                     grad = word_embeddings_weight.grad
                 torch.distributed.all_reduce(grad, group=parallel_state.get_embedding_group())
 
-    def validation_step(self, batch, batch_idx):
-        batch_for_pipeline = self.process_batch(batch)
+    def validation_step(self, dataloader_iter, batch_idx):
         tensor_shape = [self.cfg.encoder_seq_length, self.cfg.micro_batch_size, self.cfg.hidden_size]
 
         fwd_bwd_function = self._get_fwd_bwd_function()
 
         losses_reduced_per_micro_batch = fwd_bwd_function(
             forward_step_func=self.get_forward_output_and_loss_func(),
-            batch=batch_for_pipeline,
+            batch=dataloader_iter,
             model=self.model,
             forward_only=True,
             tensor_shape=tensor_shape,
@@ -422,7 +460,7 @@ class MegatronBertModel(MegatronBaseModel):
 
         torch.distributed.broadcast(averaged_loss, get_last_rank())
 
-        self.log('val_loss', averaged_loss, prog_bar=True)
+        self.log('val_loss', averaged_loss, prog_bar=True, batch_size=1)
 
     def test_step(self, batch, batch_idx):
         return self.validation_step(batch, batch_idx)
@@ -461,17 +499,6 @@ class MegatronBertModel(MegatronBaseModel):
             #     [lm_loss])
             # return loss, {'lm loss': averaged_losses[0]}
 
-    def process_batch(self, batch):
-        """Build the batch."""
-        # Unpack.
-        tokens = batch['text'].long()
-        types = batch['types'].long()
-        sentence_order = batch['is_random'].long()
-        loss_mask = batch['loss_mask'].float()
-        lm_labels = batch['labels'].long()
-        padding_mask = batch['padding_mask'].long()
-        return [tokens, types, sentence_order, loss_mask, lm_labels, padding_mask]
-
     def build_train_valid_test_datasets(self):
         logging.info('Building Bert datasets.')
         if self.trainer.limit_val_batches > 1.0 and isinstance(self.trainer.limit_val_batches, float):
@@ -621,7 +648,7 @@ class MegatronBertModel(MegatronBaseModel):
         # Megatron sampler
         if hasattr(self.cfg.data, 'dataloader_type') and self.cfg.data.dataloader_type is not None:
             if self.cfg.data.dataloader_type == 'single':
-                batch_sampler = MegatronPretrainingBatchSampler(
+                batch_sampler = MegatronPretrainingSampler(
                     total_samples=len(dataset),
                     consumed_samples=consumed_samples,
                     micro_batch_size=self.cfg.micro_batch_size,
@@ -631,11 +658,10 @@ class MegatronBertModel(MegatronBaseModel):
                     drop_last=self.cfg.get('drop_last', True),
                 )
             elif self.cfg.data.dataloader_type == 'cyclic':
-                batch_sampler = MegatronPretrainingRandomBatchSampler(
+                batch_sampler = MegatronPretrainingRandomSampler(
                     total_samples=len(dataset),
                     consumed_samples=consumed_samples,
                     micro_batch_size=self.cfg.micro_batch_size,
-                    global_batch_size=self.cfg.global_batch_size,
                     data_parallel_rank=parallel_state.get_data_parallel_rank(),
                     data_parallel_size=parallel_state.get_data_parallel_world_size(),
                     drop_last=self.cfg.get('drop_last', True),
@@ -843,3 +869,41 @@ class MegatronBertModel(MegatronBaseModel):
                 parallel_state.set_virtual_pipeline_model_parallel_rank(i)
                 self.model[i].module.load_state_dict(checkpoint[f'model{i}'], strict=True)
             parallel_state.set_virtual_pipeline_model_parallel_rank(0)
+
+    def on_train_batch_end(self, outputs, dataloader_iter: Any, batch_idx: int, unused: Optional[int] = 0) -> None:
+        super().on_train_batch_end(outputs, dataloader_iter, batch_idx)
+
+        # TODO: Replace with newer override for scheduler.step() instead of
+        # search for plugins for fp16 GradScalar
+        if self.trainer.precision_plugin is not None and isinstance(
+            self.trainer.precision_plugin, NativeMixedPrecisionPlugin
+        ):
+            precision_plugin = self.trainer.precision_plugin
+
+            if (
+                hasattr(precision_plugin, 'scaler')
+                and precision_plugin.scaler is not None
+                and isinstance(precision_plugin.scaler, GradScaler)
+            ):
+                grad_scaler = precision_plugin.scaler
+
+                # If the grad scaler skipped its optimizer step due to infs/nans,
+                # decrement the step of all schedulers.
+                if grad_scaler.optimizer_update_skipped is not None and grad_scaler.optimizer_update_skipped is True:
+                    scheduler_cfgs = self.trainer.lr_scheduler_configs
+
+                    if not scheduler_cfgs or not self.trainer.lightning_module.automatic_optimization:
+                        return
+
+                    for scheduler_cfg in scheduler_cfgs:
+                        # Decrement the counter by 2, then perform a scheduler.step() to perform a no-up
+                        # as well as update the optimizer lr in all param groups
+                        scheduler_cfg.scheduler.last_epoch -= 2
+                        scheduler_cfg.scheduler.step()
+
+                    # Removing the line below because it messes up train_valid_test_num_samples calculation.
+                    # self.trainer.fit_loop.max_steps = self.trainer.fit_loop.max_steps + 1
+
+                    # Reset the optimizer update skipped to `None` - this is to prevent scheduler no-ops during
+                    # accumulated gradient updates.
+                    grad_scaler.optimizer_update_skipped = None
