@@ -21,7 +21,7 @@ from nemo.collections.common.data import ConcatMapDataset
 from nemo.collections.common.metrics import MetricStringToTorchMetric
 from nemo.collections.common.metrics.classification_accuracy import ExactStringPerCategoryMatchMetric
 from nemo.collections.nlp.data.common.sequence_to_sequence_dataset import SequenceToSequenceDataset
-from nemo.collections.nlp.models.language_modeling.megatron_t5_model import MegatronT5Model
+from nemo.collections.nlp.models.language_modeling.megatron_t5_model import MegatronT5Model, T5Sentinel
 from nemo.collections.nlp.parts.nlp_overrides import GlobalBatchDataFetcher
 from nemo.utils import AppState, logging
 
@@ -132,22 +132,9 @@ class MegatronT5FinetuneModel(MegatronT5Model):
             self.setup_training_data()
 
     def _process_global_batch(self, global_batch):
-        """Process a list of microbatches into a global batch."""
-        # If there is no language information in the global batch (ex: English MNLI), we can use the parent global batch processor as is.
-        if 'lang' not in global_batch[0]:
-            return self._process_global_batch_without_megatron_batch_sampler(global_batch)
-
-        # For validation data (XNLI), we need to process the global batch and and then deal with language info separately.
-        else:
-            assert all(['lang' in micro_batch for micro_batch in global_batch])
-            langs_list = []
-            processed_global_batch = self._process_global_batch_without_megatron_batch_sampler(
-                [{k: v for k, v in micro_batch.items() if k != 'lang'} for micro_batch in global_batch]
-            )
-            for micro_batch in global_batch:
-                langs_list.extend(micro_batch['lang'])
-            processed_global_batch['lang'] = langs_list
-            return processed_global_batch
+        """Optionally processes a global batch."""
+        # TODO: maybe remove this now that we've refactored data batch sizes.
+        return global_batch
 
     def on_validation_epoch_start(self):
         app_state = AppState()
@@ -160,7 +147,26 @@ class MegatronT5FinetuneModel(MegatronT5Model):
         )
         return super().on_validation_epoch_start()
 
+    def on_test_epoch_start(self):
+        app_state = AppState()
+        _reconfigure_microbatch_calculator(
+            rank=app_state.global_rank,
+            rampup_batch_size=None,
+            global_batch_size=self.cfg.data.test_ds.global_batch_size,
+            micro_batch_size=self.cfg.data.test_ds.micro_batch_size,
+            data_parallel_size=parallel_state.get_data_parallel_world_size(),
+        )
+        return super().on_test_epoch_start()
+
+    def on_test_epoch_end(self):
+        self.on_inference_epoch_end(self.cfg.data.test_ds)
+        return super().on_test_epoch_end()
+
     def on_validation_epoch_end(self):
+        self.on_inference_epoch_end(self.cfg.data.validation_ds)
+        return super().on_validation_epoch_end()
+
+    def on_inference_epoch_end(self, ds):
         app_state = AppState()
         if hasattr(self, "_train_ds"):
             _reconfigure_microbatch_calculator(
@@ -176,30 +182,32 @@ class MegatronT5FinetuneModel(MegatronT5Model):
             _reconfigure_microbatch_calculator(
                 rank=app_state.global_rank,
                 rampup_batch_size=None,
-                global_batch_size=self.cfg.data.validation_ds.global_batch_size,
-                micro_batch_size=self.cfg.data.validation_ds.micro_batch_size,
+                global_batch_size=ds.global_batch_size,
+                micro_batch_size=ds.micro_batch_size,
                 data_parallel_size=parallel_state.get_data_parallel_world_size(),
             )
 
-        return super().on_validation_epoch_end()
+    def on_train_epoch_start(self) -> None:
+        # Same logic as validation epoch end, but this may be need if there is no validation sanity check to trigger validation_epoch_end()
+        self.on_validation_epoch_end()
+        return super().on_train_epoch_start()
 
     def training_step(self, batch, batch_idx):
-        micro_batch_size = batch[0]['text_enc'].size(0)
+        global_batch_size_per_gpu = batch['text_enc'].size(0)
         # This should happen only on the last batch of the dataset.
-        if micro_batch_size != self.cfg.data.train_ds.micro_batch_size:
+        if (
+            global_batch_size_per_gpu
+            != self.cfg.data.train_ds.global_batch_size // parallel_state.get_data_parallel_world_size()
+        ):
+            # NOTE: This should never really be called since `drop_last=True` is required for training datasets.
             app_state = AppState()
             _reconfigure_microbatch_calculator(
                 rank=app_state.global_rank,
                 rampup_batch_size=None,
-                global_batch_size=micro_batch_size
-                * parallel_state.get_data_parallel_world_size()
-                * get_num_microbatches(),
-                micro_batch_size=micro_batch_size,
+                global_batch_size=global_batch_size_per_gpu * parallel_state.get_data_parallel_world_size(),
+                micro_batch_size=global_batch_size_per_gpu // get_num_microbatches(),
                 data_parallel_size=parallel_state.get_data_parallel_world_size(),
             )
-        # At this point batch is a list of dictionaries where eatch dict is a microbatch.
-        # After the process_global_batch call, batch will be a single dictionary containing the global batch.
-        # This is required since the parent class expects a single global batch dictioanry.
         batch = self._process_global_batch(batch)
         return super().training_step(batch, batch_idx)
 
@@ -258,27 +266,30 @@ class MegatronT5FinetuneModel(MegatronT5Model):
 
         return pred, label
 
-    def inference_step(self, batch, batch_idx, mode, dataloader_idx=0):
-        batch_has_lang_information = len(batch[0]) == 7
-
-        micro_batch_size = batch[0]['text_enc'].size(0)
+    def _reconfigure_and_process_inference_batch(self, batch, ds_config):
+        global_batch_size_per_gpu = batch['text_enc'].size(0)
         # This should happen only on the last batch of the dataset.
-        if micro_batch_size != self.cfg.data.validation_ds.micro_batch_size:
+        if global_batch_size_per_gpu != ds_config.global_batch_size // parallel_state.get_data_parallel_world_size():
+            # NOTE: This is reconfiguring to make sure there is no grad-acc for validation batches.
             app_state = AppState()
             _reconfigure_microbatch_calculator(
                 rank=app_state.global_rank,
                 rampup_batch_size=None,
-                global_batch_size=micro_batch_size
-                * parallel_state.get_data_parallel_world_size()
-                * get_num_microbatches(),
-                micro_batch_size=micro_batch_size,
+                global_batch_size=global_batch_size_per_gpu * parallel_state.get_data_parallel_world_size(),
+                micro_batch_size=global_batch_size_per_gpu,
                 data_parallel_size=parallel_state.get_data_parallel_world_size(),
             )
 
-        # At this point processed_batch is a list of dictionaries where eatch dict is a microbatch.
-        # After the process_global_batch call, processed_batch will be a single dictionary containing the global batch.
-        # This is required since the parent class expects a single global batch dictioanry.
         processed_batch = self._process_global_batch(batch)
+        return processed_batch
+
+    def inference_step(self, batch, batch_idx, mode, dataloader_idx=0):
+        # Regular finetuning datasets will return a list of dicts for each microbatch. But T0 datasets will return a single dict for the global batch.
+        batch_has_lang_information = isinstance(batch, list) and len(batch[0]) == 7
+
+        processed_batch = self._reconfigure_and_process_inference_batch(
+            batch, self.cfg.data.validation_ds if mode == 'validation' else self.cfg.data.test_ds
+        )
 
         # Call parent validation step to get the loss.
         # NOTE: There could be extra keys in the processed_batch dictionary such as "langs" for XNLI, this will be ignored in the parent class.
@@ -289,9 +300,9 @@ class MegatronT5FinetuneModel(MegatronT5Model):
         )
 
         # Special ids to text function to handle stripping <eos> and special tokens with sentencepiece tokenizers.
-        preds_text = self.ids_to_text(predicted_token_ids)
-        labels_text = self.ids_to_text(processed_batch['labels'])
-        input_text = self.ids_to_text(processed_batch['text_enc'])
+        preds_text = MegatronT5FinetuneModel.ids_to_text(predicted_token_ids, self.tokenizer)
+        labels_text = MegatronT5FinetuneModel.ids_to_text(processed_batch['labels'], self.tokenizer)
+        input_text = MegatronT5FinetuneModel.ids_to_text(processed_batch['text_enc'], self.tokenizer)
 
         if not batch_has_lang_information:
             categories = [None] * len(preds_text)
@@ -306,8 +317,12 @@ class MegatronT5FinetuneModel(MegatronT5Model):
                 pred=pred,
                 label=label,
                 metric_name=self.val_metric_name if mode == 'validation' else self.test_metric_name,
-                class_labels=self.cfg.data.validation_ds.metric.get('class_labels', None),
-                labels_are_strings=self.cfg.data.validation_ds.metric.get('labels_are_strings', False),
+                class_labels=self.cfg.data.validation_ds.metric.get('class_labels', None)
+                if mode == 'validation'
+                else self.cfg.data.test_ds.metric.get('class_labels', None),
+                labels_are_strings=self.cfg.data.validation_ds.metric.get('labels_are_strings', False)
+                if mode == 'validation'
+                else self.cfg.data.test_ds.metric.get('labels_are_strings', False),
             )
             if batch_has_lang_information:
                 _ = metric(pred, label, category)
@@ -322,18 +337,26 @@ class MegatronT5FinetuneModel(MegatronT5Model):
             'inputs': input_text,
         }
 
-    def ids_to_text(self, batch_ids):
+    @classmethod
+    def ids_to_text(cls, batch_ids, tokenizer):
         batch_ids = batch_ids.cpu().numpy().tolist()
         texts = []
         for ids in batch_ids:
-            if self.tokenizer.eos_id in ids:
-                idx = ids.index(self.tokenizer.eos_id)
+            if tokenizer.eos_id in ids:
+                idx = ids.index(tokenizer.eos_id)
+                ids = ids[:idx]
+
+            if (
+                len(tokenizer.text_to_ids(T5Sentinel.END.value)) == 1
+                and tokenizer.text_to_ids(T5Sentinel.END.value)[0] in ids
+            ):
+                idx = ids.index(tokenizer.text_to_ids(T5Sentinel.END.value)[0])
                 ids = ids[:idx]
 
             # Legacy sentencepiece detokenization still preserves special tokens which messes up exact string match.
-            if hasattr(self.tokenizer, 'special_token_to_id'):
-                ids = [id for id in ids if id not in self.tokenizer.special_token_to_id.values()]
-            text = self.tokenizer.ids_to_text(ids)
+            if hasattr(tokenizer, 'special_token_to_id'):
+                ids = [id for id in ids if id not in tokenizer.special_token_to_id.values()]
+            text = tokenizer.ids_to_text(ids)
             texts.append(text)
 
         return texts
@@ -404,7 +427,7 @@ class MegatronT5FinetuneModel(MegatronT5Model):
                     )
 
                 # Gather the outputs object from all data parallel ranks since we are using the DistributedSampler which splits data across DDP ranks.
-                gathered_outputs = [None for _ in range(self.world_size)]
+                gathered_outputs = [None for _ in range(parallel_state.get_data_parallel_world_size())]
                 torch.distributed.all_gather_object(
                     gathered_outputs,
                     [
@@ -416,6 +439,7 @@ class MegatronT5FinetuneModel(MegatronT5Model):
                         }
                         for x in output
                     ],
+                    group=parallel_state.get_data_parallel_group(),
                 )
 
                 # Figure out what the suffix of the file should be.
@@ -432,7 +456,7 @@ class MegatronT5FinetuneModel(MegatronT5Model):
 
                 # PTL models have a self.global_rank attribute and we want to write to disk only on global rank 0.
                 if self.global_rank == 0:
-                    for rank in range(0, self.world_size):
+                    for rank in range(0, parallel_state.get_data_parallel_world_size()):
                         for batch in gathered_outputs[rank]:
                             for pred, label, input, category in zip(
                                 batch['preds'], batch['labels'], batch['inputs'], batch['categories']
@@ -468,13 +492,10 @@ class MegatronT5FinetuneModel(MegatronT5Model):
         return averaged_loss, averaged_metric
 
     def write_predictions_to_file(self, outputs, output_file_path_prefix):
-        with open(output_file_path_prefix + "_inputs_preds_labels.json", "w") as f_json:
-            json_output = {
-                "inputs": outputs["inputs"],
-                "preds": outputs["preds"],
-                "labels": outputs["labels"],
-            }
-            json.dump(json_output, f_json)
+        with open(output_file_path_prefix + "_inputs_preds_labels.jsonl", "w") as f_json:
+            assert len(outputs['inputs']) == len(outputs['preds']) == len(outputs['labels'])
+            for i, p, l in zip(outputs['inputs'], outputs['preds'], outputs['labels']):
+                f_json.write(json.dumps({'input': i, 'pred': p, 'label': l}) + '\n')
 
     def validation_step(self, batch, batch_idx, dataloader_idx=0):
         return self.inference_step(batch, batch_idx, 'validation', dataloader_idx)
@@ -489,15 +510,7 @@ class MegatronT5FinetuneModel(MegatronT5Model):
         _ = self.inference_epoch_end(outputs, 'test', self.cfg.data.test_ds)
 
     def build_data_loader(
-        self,
-        dataset,
-        micro_batch_size,
-        global_batch_size,
-        shuffle,
-        num_workers,
-        pin_memory,
-        drop_last,
-        check_validation_interval,
+        self, dataset, global_batch_size, shuffle, num_workers, pin_memory, drop_last,
     ):
         """Buld dataloader given an input dataset."""
 
@@ -509,20 +522,6 @@ class MegatronT5FinetuneModel(MegatronT5Model):
         sampler = torch.utils.data.distributed.DistributedSampler(
             dataset, num_replicas=world_size, rank=rank, shuffle=shuffle
         )
-        # This check makes sure the val_check_interval is less than the number of global batches.
-        # Normally, PTL would do this check and properly account for gradient accumulation.
-        # But now, it is implicit in the apex fwd/bwd functions and so we need to check for this somewhere.
-        # The consequence of not doing this is that training loop will never run validation.
-        # NOTE: Prog bar is also broken as a result of this.
-        global_batch_size_per_gpu = micro_batch_size * get_num_microbatches()
-        if (
-            self.trainer.val_check_interval > (sampler.num_samples // global_batch_size_per_gpu)
-            and check_validation_interval
-        ):
-            raise ValueError(
-                f"trainer.val_check_interval {self.trainer.val_check_interval} is > number of global batches {sampler.num_samples // global_batch_size}"
-            )
-
         if isinstance(dataset, ConcatMapDataset):
             collate_fn = dataset.datasets[0].collate_fn
         else:
@@ -532,7 +531,7 @@ class MegatronT5FinetuneModel(MegatronT5Model):
             dataset,
             collate_fn=collate_fn,
             sampler=sampler,
-            batch_size=micro_batch_size,
+            batch_size=global_batch_size // parallel_state.get_data_parallel_world_size(),
             num_workers=num_workers,
             pin_memory=pin_memory,
             drop_last=drop_last,
@@ -541,13 +540,11 @@ class MegatronT5FinetuneModel(MegatronT5Model):
     def setup_training_data(self):
         self._train_dl = self.build_data_loader(
             self._train_ds,
-            micro_batch_size=self.cfg.data.train_ds.micro_batch_size,
             global_batch_size=self.cfg.data.train_ds.global_batch_size,
             shuffle=self.cfg.data.train_ds.shuffle,
             num_workers=self.cfg.data.train_ds.num_workers,
             pin_memory=self.cfg.data.train_ds.pin_memory,
             drop_last=self.cfg.data.train_ds.drop_last,
-            check_validation_interval=True,
         )
 
     def setup_eval_data(self, datasets, data_cfg):
@@ -555,13 +552,11 @@ class MegatronT5FinetuneModel(MegatronT5Model):
         for dataset in datasets:
             eval_dl = self.build_data_loader(
                 dataset,
-                micro_batch_size=data_cfg.micro_batch_size,
                 global_batch_size=data_cfg.global_batch_size,
                 shuffle=data_cfg.shuffle,
                 num_workers=data_cfg.num_workers,
                 pin_memory=data_cfg.pin_memory,
                 drop_last=data_cfg.drop_last,
-                check_validation_interval=False,
             )
             dataloaders.append(eval_dl)
         return dataloaders
@@ -603,6 +598,9 @@ class MegatronT5FinetuneModel(MegatronT5Model):
                 tgt_tokenizer=self.tokenizer,
                 max_src_seq_length=data_cfg.max_src_seq_length,
                 max_tgt_seq_length=data_cfg.max_tgt_seq_length,
+                add_bos_to_input=data_cfg.get('add_bos_to_input', True),
+                add_eos_to_input=data_cfg.get('add_eos_to_input', True),
+                replace_bos_with_pad=data_cfg.get('replace_bos_with_pad', False),
             )
             datasets.append(dataset)
 
@@ -655,6 +653,9 @@ class MegatronT5FinetuneModel(MegatronT5Model):
                 tgt_tokenizer=self.tokenizer,
                 max_src_seq_length=data_cfg.max_src_seq_length,
                 max_tgt_seq_length=data_cfg.max_tgt_seq_length,
+                add_bos_to_input=data_cfg.get('add_bos_to_input', True),
+                add_eos_to_input=data_cfg.get('add_eos_to_input', True),
+                replace_bos_with_pad=data_cfg.get('replace_bos_with_pad', False),
             )
             datasets.append(dataset)
 
@@ -673,15 +674,3 @@ class MegatronT5FinetuneModel(MegatronT5Model):
             return
         self._train_ds = self._build_train_dataset(self.cfg.data.train_ds)
         logging.info(f'Finished building datasets ...')
-
-    def on_train_start(self) -> None:
-        """PTL hook used to override DataFetcher with GlobalBatchDataFetcher """
-        self.trainer.fit_loop._data_fetcher = GlobalBatchDataFetcher()
-
-    def on_validation_start(self) -> None:
-        """PTL hook used to override DataFetcher with GlobalBatchDataFetcher """
-        self.trainer.fit_loop.epoch_loop.val_loop._data_fetcher = GlobalBatchDataFetcher()
-        self.trainer.validate_loop._data_fetcher = GlobalBatchDataFetcher()
-
-    def on_test_start(self) -> None:
-        self.trainer.test_loop._data_fetcher = GlobalBatchDataFetcher()
