@@ -80,7 +80,7 @@ class MegatronGPTBaseAdapterModel(MegatronGPTPromptLearningModel):
         return output
 
     def setup(self, stage=None):
-        if stage == 'predict' or self.virtual_prompt_style == VirtualPromptStyle.INFERENCE:
+        if stage == 'predict':
             self.frozen_model.freeze()
             return
 
