@@ -12,15 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from nemo.collections.tts.modules.degli import DegliModule
-from nemo.collections.tts.modules.ed_mel2spec import EDMel2SpecModule
-from nemo.collections.tts.modules.glow_tts import GlowTTSModule
-from nemo.collections.tts.modules.melgan_modules import (
-    MelGANDiscriminator,
-    MelGANGenerator,
-    MelGANMultiScaleDiscriminator,
-)
-from nemo.collections.tts.modules.squeezewave import SqueezeWaveModule
 from nemo.collections.tts.modules.tacotron2 import Decoder as Taco2Decoder
 from nemo.collections.tts.modules.tacotron2 import Encoder as Taco2Encoder
 from nemo.collections.tts.modules.tacotron2 import Postnet as Taco2Postnet
