@@ -16,6 +16,8 @@ from nemo.core.classes.mixins.access_mixins import AccessMixin, set_access_cfg
 from nemo.core.classes.mixins.adapter_mixin_strategies import (
     ResidualAddAdapterStrategy,
     ResidualAddAdapterStrategyConfig,
+    ReturnResultAdapterStrategy,
+    ReturnResultAdapterStrategyConfig,
 )
 from nemo.core.classes.mixins.adapter_mixins import (
     AdapterModelPTMixin,
