@@ -43,11 +43,11 @@ from nemo.collections.common.callbacks import EMA
 from nemo.constants import NEMO_ENV_VARNAME_TESTING, NEMO_ENV_VARNAME_VERSION
 from nemo.utils import logging, timers
 from nemo.utils.app_state import AppState
-from nemo.utils.dllogger import DLLogger
 from nemo.utils.env_var_parsing import get_envbool
 from nemo.utils.exceptions import NeMoBaseException
 from nemo.utils.get_rank import is_global_rank_zero
 from nemo.utils.lightning_logger_patch import add_filehandlers_to_pl_logger
+from nemo.utils.loggers import ClearMLLogger, ClearMLParams, DLLogger, DLLoggerParams, MLFlowParams
 from nemo.utils.model_utils import inject_model_parallel_rank, uninject_model_parallel_rank
 
 
@@ -106,28 +106,6 @@ class CallbackParams:
     save_on_train_epoch_end: Optional[bool] = False  # Save after training, not after validation
 
 
-@dataclass
-class MLFlowParams:
-    # name of experiment, if none, defaults to the globally set experiment name
-    experiment_name: Optional[str] = None
-    # no run_name because it's set by version
-    # local or remote tracking seerver. If tracking_uri is not set, it defaults to save_dir
-    tracking_uri: Optional[str] = None
-    tags: Optional[Dict[str, Any]] = None
-    save_dir: Optional[str] = "./mlruns"
-    prefix: str = ""
-    artifact_location: Optional[str] = None
-    # provide run_id if resuming a previously started run
-    run_id: Optional[str] = None
-
-
-@dataclass
-class DLLoggerParams:
-    verbose: Optional[bool] = False
-    stdout: Optional[bool] = False
-    json_file: Optional[str] = "./dllogger.json"
-
-
 @dataclass
 class StepTimingParams:
     reduction: Optional[str] = "mean"
@@ -148,6 +126,9 @@ class EMAParams:
 
 @dataclass
 class ExpManagerConfig:
+    """Experiment Manager config for validation of passed arguments.
+    """
+
     # Log dir creation parameters
     explicit_log_dir: Optional[str] = None
     exp_dir: Optional[str] = None
@@ -166,6 +147,8 @@ class ExpManagerConfig:
     mlflow_logger_kwargs: Optional[MLFlowParams] = MLFlowParams()
     create_dllogger_logger: Optional[bool] = False
     dllogger_logger_kwargs: Optional[DLLoggerParams] = DLLoggerParams()
+    create_clearml_logger: Optional[bool] = False
+    clearml_logger_kwargs: Optional[ClearMLParams] = ClearMLParams()
     # Checkpointing parameters
     create_checkpoint_callback: Optional[bool] = True
     checkpoint_callback_params: Optional[CallbackParams] = CallbackParams()
@@ -239,7 +222,8 @@ def exp_manager(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictCo
     directory. exp_manager also allows for explicit folder creation via explicit_log_dir.
 
     The version can be a datetime string or an integer. Datestime version can be disabled if use_datetime_version is set
-     to False. It optionally creates TensorBoardLogger, WandBLogger, DLLogger, MLFlowLogger, ModelCheckpoint objects from pytorch lightning.
+    to False. It optionally creates TensorBoardLogger, WandBLogger, DLLogger, MLFlowLogger, ClearMLLogger,
+    ModelCheckpoint objects from pytorch lightning.
     It copies sys.argv, and git information if available to the logging directory. It creates a log file for each
     process to log their output into.
 
@@ -286,6 +270,9 @@ def exp_manager(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictCo
             - create_dllogger_logger (bool): Whether to create an DLLogger logger and attach it to the pytorch lightning
                 training. Defaults to False
             - dllogger_logger_kwargs (dict): optional parameters for the DLLogger logger
+            - create_clearml_logger (bool): Whether to create an ClearML logger and attach it to the pytorch lightning
+                training. Defaults to False
+            - clearml_logger_kwargs (dict): optional parameters for the ClearML logger
             - create_checkpoint_callback (bool): Whether to create a ModelCheckpoint callback and attach it to the
                 pytorch lightning trainer. The ModelCheckpoint saves the top 3 models with the best "val_loss", the most
                 recent checkpoint under ``*last.ckpt``, and the final checkpoint after training completes under ``*end.ckpt``.
@@ -410,12 +397,15 @@ def exp_manager(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictCo
         or cfg.create_wandb_logger
         or cfg.create_mlflow_logger
         or cfg.create_dllogger_logger
+        or cfg.create_clearml_logger
     ):
         configure_loggers(
             trainer,
             exp_dir,
+            log_dir,
             cfg.name,
             cfg.version,
+            cfg.checkpoint_callback_params,
             cfg.create_tensorboard_logger,
             cfg.summary_writer_kwargs,
             cfg.create_wandb_logger,
@@ -424,6 +414,8 @@ def exp_manager(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictCo
             cfg.mlflow_logger_kwargs,
             cfg.create_dllogger_logger,
             cfg.dllogger_logger_kwargs,
+            cfg.create_clearml_logger,
+            cfg.clearml_logger_kwargs,
         )
 
     # add loggers timing callbacks
@@ -762,8 +754,10 @@ def get_git_diff():
 def configure_loggers(
     trainer: 'pytorch_lightning.Trainer',
     exp_dir: [Path, str],
+    log_dir: [Path, str],
     name: str,
     version: str,
+    checkpoint_callback_params: dict,
     create_tensorboard_logger: bool,
     summary_writer_kwargs: dict,
     create_wandb_logger: bool,
@@ -772,9 +766,11 @@ def configure_loggers(
     mlflow_kwargs: dict,
     create_dllogger_logger: bool,
     dllogger_kwargs: dict,
+    create_clearml_logger: bool,
+    clearml_kwargs: dict,
 ):
     """
-    Creates TensorboardLogger and/or WandBLogger / MLFlowLogger / DLlogger and attach them to trainer.
+    Creates TensorboardLogger and/or WandBLogger / MLFlowLogger / DLlogger / ClearMLLogger and attach them to trainer.
     Raises ValueError if summary_writer_kwargs or wandb_kwargs are misconfigured.
     """
     # Potentially create tensorboard logger and/or WandBLogger / MLFlowLogger / DLLogger
@@ -818,6 +814,17 @@ def configure_loggers(
         logger_list.append(dllogger_logger)
         logging.info("DLLogger has been set up")
 
+    if create_clearml_logger:
+        clearml_logger = ClearMLLogger(
+            clearml_cfg=clearml_kwargs,
+            log_dir=log_dir,
+            prefix=name,
+            save_best_model=checkpoint_callback_params.save_best_model,
+        )
+
+        logger_list.append(clearml_logger)
+        logging.info("ClearMLLogger has been set up")
+
     trainer._logger_connector.configure_logger(logger_list)
 
 
@@ -920,38 +927,47 @@ class NeMoModelCheckpoint(ModelCheckpoint):
         self.best_model_score = self.best_k_models[self.best_model_path]
 
     def on_save_checkpoint(self, trainer, pl_module, checkpoint):
-        # output = None
         output = super().on_save_checkpoint(trainer, pl_module, checkpoint)
         if not self.always_save_nemo:
             return output
+        # Load the best model and then re-save it
+        app_state = AppState()
+        if app_state.model_parallel_size is not None and app_state.model_parallel_size > 1:
+            logging.warning(f'always_save_nemo will slow down training for model_parallel > 1.')
+        # since we are creating tarfile artifacts we need to update .nemo path
+        app_state.model_restore_path = os.path.abspath(
+            os.path.expanduser(os.path.join(self.dirpath, self.prefix + self.postfix))
+        )
+        if app_state.model_parallel_size is not None and app_state.model_parallel_size > 1:
+            maybe_injected_best_model_path = inject_model_parallel_rank(self.best_model_path)
         else:
-            # Load the best model and then re-save it
-            app_state = AppState()
-            if app_state.model_parallel_size is not None and app_state.model_parallel_size > 1:
-                raise ValueError(f'always_save_nemo is not implemented for model parallel models.')
-            # since we are creating tarfile artifacts we need to update .nemo path
-            app_state.model_restore_path = os.path.abspath(
-                os.path.expanduser(os.path.join(self.dirpath, self.prefix + self.postfix))
-            )
-            if self.save_best_model:
-                if not os.path.exists(self.best_model_path):
-                    return output
+            maybe_injected_best_model_path = self.best_model_path
 
-                if self.best_model_path == self.previous_best_path:
-                    return output
+        if self.save_best_model:
+            if not os.path.exists(maybe_injected_best_model_path):
+                return
 
-                self.previous_model_path = self.best_model_path
-                old_state_dict = deepcopy(pl_module.state_dict())
-                checkpoint = torch.load(self.best_model_path, map_location='cpu')
-                if 'state_dict' in checkpoint:
-                    checkpoint = checkpoint['state_dict']
-                # get a new instanace of the model
-                pl_module.load_state_dict(checkpoint, strict=True)
-                pl_module.save_to(save_path=app_state.model_restore_path)
-                pl_module.load_state_dict(old_state_dict, strict=True)
-            else:
-                pl_module.save_to(save_path=app_state.model_restore_path)
-            return output
+            if self.best_model_path == self.previous_best_path:
+                return output
+
+            self.previous_model_path = self.best_model_path
+            old_state_dict = deepcopy(pl_module.state_dict())
+            checkpoint = torch.load(maybe_injected_best_model_path, map_location='cpu')
+            if 'state_dict' in checkpoint:
+                checkpoint = checkpoint['state_dict']
+            # get a new instanace of the model
+            pl_module.load_state_dict(checkpoint, strict=True)
+            if torch.distributed.is_initialized():
+                torch.distributed.barrier()
+            pl_module.save_to(save_path=app_state.model_restore_path)
+            logging.info(f"New best .nemo model saved to: {app_state.model_restore_path}")
+            pl_module.load_state_dict(old_state_dict, strict=True)
+        else:
+            if torch.distributed.is_initialized():
+                torch.distributed.barrier()
+            pl_module.save_to(save_path=app_state.model_restore_path)
+            logging.info(f"New .nemo model saved to: {app_state.model_restore_path}")
+        return output
 
     def on_train_end(self, trainer, pl_module):
         if trainer.fast_dev_run:
