@@ -46,6 +46,25 @@ _EXT_DICT = {
 }
 
 
+class TorchRMSNorm(nn.Module):
+    def __init__(self, weight, eps=1e-6):
+        """
+        LayerNorm without bias
+        """
+        super().__init__()
+        self.weight = weight
+        self.variance_epsilon = eps
+
+    def forward(self, hidden_states):
+        # can be only calculated with precision=32
+        variance = hidden_states.to(torch.float32).pow(2).mean(-1, keepdim=True)
+        hidden_states = hidden_states * torch.rsqrt(variance + self.variance_epsilon)
+        if self.weight.dtype in [torch.float16, torch.bfloat16]:
+            hidden_states = hidden_states.to(self.weight.dtype)
+
+        return self.weight * hidden_states
+
+
 class LinearWithBiasSkip(nn.Module):
     def __init__(self, weight, bias, skip_bias_add):
         super(LinearWithBiasSkip, self).__init__()
@@ -108,11 +127,12 @@ def parse_input_example(input_example):
 def to_onnxrt_input(ort_input_names, input_names, input_dict, input_list):
     odict = {}
     for k in reversed(input_names):
+        val = None
         if k in input_dict:
             val = input_dict[k].cpu().numpy()
-        else:
+        elif len(input_list) > 0:
             val = input_list.pop().cpu().numpy()
-        if k in ort_input_names:
+        if k in ort_input_names and val is not None:
             odict[k] = val
     return odict
 
@@ -208,6 +228,7 @@ apex_available = True
 
 try:
     from apex.contrib.layer_norm.layer_norm import FastLayerNorm
+    from apex.normalization import MixedFusedRMSNorm
     from apex.normalization.fused_layer_norm import FusedLayerNorm, MixedFusedLayerNorm
     from apex.transformer.functional.fused_softmax import FusedScaleMaskSoftmax
     from apex.transformer.tensor_parallel.layers import ColumnParallelLinear, RowParallelLinear
@@ -222,6 +243,7 @@ try:
         """
 
         p = next(n.parameters())
+
         if isinstance(n, FusedLayerNorm) or isinstance(n, MixedFusedLayerNorm):
             shape, eps, affine = n.normalized_shape, n.eps, n.elementwise_affine
         elif isinstance(n, FastLayerNorm):
@@ -229,27 +251,29 @@ try:
         else:
             return None
 
-        mod = nn.LayerNorm(shape, eps=eps, elementwise_affine=affine, device=p.device, dtype=p.dtype)
         n_state = n.state_dict()
+        mod = nn.LayerNorm(shape, eps=eps, elementwise_affine=affine, device=p.device, dtype=p.dtype)
+
         mod.load_state_dict(n_state)
+
         return mod
 
-    def replace_RowParallelLinear(n: nn.Module) -> Optional[nn.Linear]:
+    def replace_MixedFusedRMSNorm(n: nn.Module):
         """
-        Replaces Apex's FusedLayerNorm with nn.LayerNorm. This is required for ONNX export.
+        Replaces Apex's MixedFusedRMSNorm with equivalent Pytorch layer. This is required for ONNX export.
         Args:
-           n: the FusedLayerNorm pytorch module to replace
+           n: the MixedFusedRMSNorm pytorch module to replace
         Returns:
-           Equivalent LayerNorm module
+           Equivalent module
         """
-        if not isinstance(n, RowParallelLinear):
-            raise ValueError("This function can only change the RowParallelLinear module.")
 
-        dev = next(n.parameters()).device
-        mod = LinearWithBiasSkip(n.weight, n.bias, n.skip_bias_add).to(device=dev)
+        p = next(n.parameters())
+
+        if isinstance(n, MixedFusedRMSNorm):
+            mod = TorchRMSNorm(n.state_dict()['weight'], n.eps).to(p.device)
+        else:
+            return None
 
-        n_state = n.state_dict()
-        mod.load_state_dict(n_state)
         return mod
 
     def replace_ParallelLinear(n: nn.Module) -> Optional[nn.Linear]:
@@ -295,6 +319,7 @@ try:
         "RowParallelLinear": replace_ParallelLinear,
         "ColumnParallelLinear": replace_ParallelLinear,
         "FusedScaleMaskSoftmax": replace_FusedScaleMaskSoftmax,
+        "MixedFusedRMSNorm": replace_MixedFusedRMSNorm,
     }
 
 except Exception as e:
